import sys
import os
from PyQt5.QtWidgets import QApplication
from xulpymoney.ui.frmInit import frmInit


def main():

    try:
        os.makedirs( os.environ['HOME']+"/.xulpymoney/tmp/")
    except:
        pass
    app = QApplication(sys.argv)
    app.setOrganizationName("Mariano Muñoz ©")
    app.setOrganizationDomain("turulomio.users.sourceforge.net")
    app.setApplicationName("Xulpymoney Init")

    frm = frmInit()
    frm.show()

    sys.exit(app.exec_())



from PyQt5.QtCore import pyqtSlot
from PyQt5.QtWidgets import QDialog, QDialogButtonBox
from xulpymoney.libxulpymoney import Split
import datetime

from xulpymoney.ui.Ui_frmSplit import Ui_frmSplit

class frmSplit(QDialog, Ui_frmSplit):
    def __init__(self, mem, product, split=None,  parent = None):
        """
        Constructor
        
        @param parent The parent widget of this dialog. (QWidget)
        @param name The name of this dialog. (QString)
        @param modal Flag indicating a modal dialog. (boolean)
        """
        QDialog.__init__(self, parent)
        self.mem=mem
        self.product=product
        self.split=split
         
        self.setModal(True)
        self.setupUi(self)
        
        self.wdgDt.blockSignals(True)
        self.wdgDt.show_microseconds(False)
        self.wdgDt.grp.setTitle(self.tr("Select the day and time of split"))
        
        if self.split==None:
            self.wdgDt.set(self.mem, datetime.datetime.now(), self.mem.localzone)
            self.split=Split(self.mem)
            self.split.product=self.product
        else:
            self.split=split
            self.wdgDt.set(self.mem, self.split.datetime, self.mem.localzone)
            self.txtInitial.setText(self.split.before)
            self.txtFinal.setText(self.split.after)
            self.txtComment.setText(self.split.comment)
        self.wdgDt.blockSignals(False)
        self.generateExample()
        
        
    def generateExample(self):
        try:
            self.split.datetime=self.wdgDt.datetime()
            self.split.after=self.txtFinal.decimal()
            self.split.before=self.txtInitial.decimal()
            self.split.comment=self.txtComment.text()
            self.lblExample.setText(self.tr("If you have 1000 shares of 10 \u20ac of price, you will have {0:.6f} shares of {1:.6f} \u20ac of price after the {2}".format(self.split.convertShares(1000),self.split.convertPrices(10),  self.split.type())))
            self.buttonbox.button(QDialogButtonBox.Ok).setEnabled(True)
        except:
            self.lblExample.setText("")
            self.buttonbox.button(QDialogButtonBox.Ok).setEnabled(False)

    def on_wdgDt_changed(self):
        self.generateExample()

    def on_txtInitial_textChanged(self):
        self.generateExample()

    def on_txtFinal_textChanged(self):
        self.generateExample()
        
    @pyqtSlot()
    def on_buttonbox_accepted(self):
        self.split.save()
        self.accept()#No haría falta pero para recordar que hay buttonbox
        
    @pyqtSlot()
    def on_buttonbox_rejected(self):
        self.reject()#No haría falta pero para recordar que hay buttonbox
    



## @brief Module with objects managers as list or as dictionary.
##
## This file is from Xulpymoney project. Do not edit, It will be overriden.
##
## You have to use list objects if you are going to make selections and secuential access.
##
## You have to use dictionary objects i f you are going to make unordered access to the dictionary. It consumes more memory. To access a selected item in a table you have to hide a column with the id and getit when selecting a row
##

import logging


## Defines who self.selected is managed
## If can take the following values
## - Object self.selected is a object
## - List self.selected is a list of objects
## - Manager. Selef selected is an object
class ManagerSelectionMode:
    Object=0
    List=1
    Manager=2

class MyMem:
    def __init__(self):
        self.mem=None
        
    def setMem(self, mem):
        self.mem=mem

class ObjectManager(object):
    def __init__(self):
        self.arr=[]       
        self.selected=None#Used to select a item in the set. Usefull in tables. Its a item

    def append(self,  obj):
        self.arr.append(obj)

    def remove(self, obj):
        self.arr.remove(obj)

    def length(self):
        return len(self.arr)
        
    #To use the same name as DictObjectManager
    def values(self):
        return self.arr

    def clean(self):
        """Deletes all items"""
        self.arr=[]
                
    def clone(self,  *initparams):
        """Returns other Set object, with items referenced, ojo con las formas de las instancias
        initparams son los parametros de iniciación de la clase"""
        result=self.__class__(*initparams)#Para que coja la clase del objeto que lo invoca
        for a in self.arr:
            result.append(a)
        return result
   
    def first(self):
        if self.length()>0:
            return self.arr[0]
        else:
            print ("There is no first item")
            return None
        
    def last(self):
        return self.arr[self.length()-1]

        
        
    def print(self):
        print ("Objects in {}".format(self.__class__))
        for q in self.arr:
            print(" * {}".format(q))


## Manager Selection class
## By default selectionmode is
class ManagerSelection(object):
    def __init__(self):
        self.__selected=None
        self.__selectionmode=ManagerSelectionMode.Object
        
    @property
    def selected(self):
        return self.__selected
        
    @selected.setter
    def selected(self, value):
        self.__selected=value
        
    def selectionMode(self):
        return self.__selectionmode
        
    def setSelectionMode(self, value):
        self.__selectionmode=value
        if value==ManagerSelectionMode.Object:
            self.selected=None
        elif value==ManagerSelectionMode.List:
            self.selected=[]
        elif value==ManagerSelectionMode.Manager:#Returns parent __class__
            self.selected=self.__class__.__bases__[0]()
        
    def cleanSelection(self):
        if self.selectionMode()==ManagerSelectionMode.Object:
            self.selected=None
        elif self.selectionMode()==ManagerSelectionMode.List:
            self.selected=[]
        elif self.selectionMode()==ManagerSelectionMode.Manager:#Returns parent __class__
            self.selected.clean()
    
    ## Useful to setselection without interactivvite ui
    ## @param list List of objects. These objects have o.id so I can append them
    def setSelected(self, list):
        self.cleanSelection()
        if self.selectionMode()==ManagerSelectionMode.List:
            for o in list:
                self.selected.append(o)
        elif self.selectionMode()==ManagerSelectionMode.Manager:
            for o in list:
                self.selected.append(o)
            

## Objects in DictListObjectManager has and id. The Id can be a integer or a string or ...
class ObjectManager_With_Id(ObjectManager):
    def __init__(self):
        ObjectManager.__init__(self)
        
    def arr_position(self, id):
        """Returns arr position of the id, useful to select items with unittests"""
        for i, a in enumerate(self.arr):
            if a.id==id:
                return i
        return None
        
    ##Returns an array with all object ids
    def array_of_ids(self):
        r=[]
        for o in self.arr:
            r.append(o.id)
        return r
    

    ## Search by id iterating array
    def find_by_id(self, id):
        for a in self.arr:
            if a.id==id:
                return a
        return None
                
    def order_by_id(self):
        """Orders the Set using self.arr"""
        try:
            self.arr=sorted(self.arr, key=lambda c: c.id,  reverse=False)     
            return True
        except:
            return False
        
    def union(self,  set,  *initparams):
        """Returns a new set, with the union comparing id
        initparams son los parametros de iniciación de la clse"""        
        resultado=self.__class__(*initparams)#Para que coja la clase del objeto que lo invoca SetProduct(self.mem), luego será self.mem
        for p in self.arr:
            resultado.append(p)
        for p in set.arr:
            if resultado.find_by_id(p.id, False)==None:
                resultado.append(p)
        return resultado

    ## Searches the objects id in the array and mak selected. ReturnsTrue if the o.id exists in the arr and False if don't
    ## It's used when I want to mark an item in a table and I only have an id
    def setSelected(self, sel):
        for i, o in enumerate(self.arr):
            if o.id==sel.id:
                self.selected=o
                return True
        self.selected=None
        return False        
        
    ## Searches the objects id in the array and mak selected. ReturnsTrue if the o.id exists in the arr and False if don't
    ## It's used when I want to mark an item in a table and I only have the list of ids
    def setSelectedList(self, lista):
        assert type(lista) is list, "id is not a list {}".format(lista)
        self.arr=[]
        for i, o in enumerate(self.arr):
            for l in lista:
                if o.id==l.id:
                    self.append(o)
        self.selected=None
        return False

## Objects in DictListObjectManager has and id and a date attribute
class ObjectManager_With_IdDate(ObjectManager_With_Id):
    def __init__(self):
        ObjectManager_With_Id.__init__(self)

    def order_by_date(self):       
        self.arr=sorted(self.arr, key=lambda e: e.date,  reverse=False) 
        
## Objects in DictListObjectManager has and id and a datetime
class ObjectManager_With_IdDatetime(ObjectManager_With_Id):
    def __init__(self):
        ObjectManager_With_Id.__init__(self)

    def order_by_datetime(self):       
        self.arr=sorted(self.arr, key=lambda e: e.datetime,  reverse=False) 
                
    ## Function that returns the same object manager, with a pointer to the of the objects that contains from the datetime given in the parameter.
    ## For example the constuctor of InvemestOperationHomogeneous is InvesmentOperationHomogeneous(mem,investment). so to use this function you need ObjectManager_from_datetime(dt,mem,investment)
    ## @param datetime. This function copies all object with datetime until this parameter
    ## @param initparams. Parameters of the constructor of the ManagerObject class
    def ObjectManager_from_datetime(self, dt, *initparams):
        result=self.__class__(*initparams)#Para que coja la clase del objeto que lo invoca
        if dt==None:
            dt=self.mem.localzone.now()
        for a in self.arr:
            if a.datetime>=dt:
                result.append(a)
        return result         
        
    ## Function that returns the same object manager, with a pointer to the of the objects that contains from the datetime given in the parameter.
    ## For example the constuctor of InvemestOperationHomogeneous is InvesmentOperationHomogeneous(mem,investment). so to use this function you need ObjectManager_from_datetime(dt,mem,investment)
    ## @param datetime. This function copies all object with datetime until this parameter
    ## @param initparams. Parameters of the constructor of the ManagerObject class
    def ObjectManager_until_datetime(self, dt, *initparams):        
        result=self.__class__(*initparams)#Para que coja la clase del objeto que lo invoca
        if dt==None:
            dt=self.mem.localzone.now()
        for a in self.arr:
            if a.datetime<=dt:
                result.append(a)
        return result
        
    def ObjectManager_copy_from_datetime(self, dt, *initparams):
        """Función que devuelve otro SetInvestmentOperations con las oper que tienen datetime mayor o igual a la pasada como parametro tambien copiadas."""
        result=self.__class__(*initparams)#Para que coja la clase del objeto que lo invoca
        if dt==None:
            dt=self.mem.localzone.now()
        for a in self.arr:
            if a.datetime>=dt:
                result.append(a.copy())
        return result
        
    ## Function that returns the same object manager, but with a copy of the objects that contains until the datetime given in the parameter.
    ## For exemple the constuctor of InvemestOperationHomogeneous is InvemestOperationHomogeneous(mem,investment). so to use this function you need ObjectManager_copy_until_datetime(dt,mem,investment)
    ## @param datetime. This function copies all object with datetime until this parameter
    ## @param initparams. Parameters of the constructor of the ManagerObject class
    def ObjectManager_copy_until_datetime(self, dt, *initparams):
        result=self.__class__(*initparams)#Para que coja la clase del objeto que lo invoca
        if dt==None:
            dt=self.mem.localzone.now()
        for a in self.arr:
            if a.datetime<=dt:
                result.append(a.copy())
        return result

## Objects in DictListObjectManager has and id and a name
class ObjectManager_With_IdName(ObjectManager_With_Id):
    def __init__(self):
        ObjectManager_With_Id.__init__(self)
        
    ## Find an object searching in its name to match the parameter
    def find_by_name(self, name,  log=False):
        """self.find_by_id() search by id (number).
        This function replaces  it and searches by name (Europe/Madrid)"""
        for a in self.arr:
            if a.name==name:
                return a
        logging.debug("{} didn't find the name: {}".format(self.__class__, name))
        return None
        
    def order_by_name(self):
        """Orders the Set using self.arr"""
        try:
            self.arr=sorted(self.arr, key=lambda c: c.name,  reverse=False)       
            return True
        except:
            return False        

    def order_by_upper_name(self):
        """Orders the Set using self.arr"""
        try:
            self.arr=sorted(self.arr, key=lambda c: c.name.upper(),  reverse=False)       
            return True
        except:
            return False

    def qcombobox(self, combo,  selected=None):
        """Load set items in a comobo using id and name
        Selected is and object
        It sorts by name the arr""" 
        self.order_by_name()
        combo.clear()
        for a in self.arr:
            combo.addItem(a.name, a.id)

        if selected!=None:
            combo.setCurrentIndex(combo.findData(selected.id))

## Objects has a field called id, whose string is the key of the item of dict
## It Can be a DictObjectManager without id
## It doesn't need to cfreate DictListObjectManager_With_IdName, because all funcions are used with ObjectManager_With_IdName
class DictObjectManager_With_Id(object):
    def __init__(self):
        self.dic={}

    def append(self,  obj):
        self.dic[str(obj.id)]=obj

    def values(self):
        return self.dic.values()

    def keys(self):
        return self.dic.keys()

    def items(self):
        return self.dic.items()

    def remove(self, obj):
        del self.dic[str(obj.id)]

    def clean(self):
        self.dic={}

    def length(self):
        return len(self.dic)

    ## Sometimes there is a dictionary with a unique valor. This function returns the value, not the key.
    ## I dón't use first because dict has no orders.
    def only(self):
        return self.dic[next(iter(self.dic))]

    ## Find by object passing o that is an object        
    def find(self, o,  log=False):
        """o is and object with id parameter"""
        print("find is deprecated")
        try:
            return self.dic[str(o.id)]    
        except:
            if log:
                print ("DictObjectManager_With_IdName ({}) fails finding {}".format(self.__class__.__name__, o.id))
            return None        

    def find_by_id(self, id,  log=False):
        """Finds by id"""
        try:
            return self.dic[str(id)]    
        except:
            if log:
                print ("DictObjectManager_With_IdName ({}) fails finding {}".format(self.__class__.__name__, id))
            return None
            
    def values_order_by_id(self):
        return sorted(self.dic.values(), key=lambda o: o.id)


class DictObjectManager_With_IdName(DictObjectManager_With_Id):
    """Base clase to create Sets, it needs id and name attributes, as index. It has a list arr and a dics dict to access objects of the set"""
    def __init__(self):
        DictObjectManager_With_Id.__init__(self)

    ## Uses dict because is faster
    def values_order_by_name(self):
        return sorted(self.dic.values(), key=lambda o: o.name)
        
class DictObjectManager_With_IdDate(DictObjectManager_With_Id):
    """Base clase to create Sets, it needs id and name attributes, as index. It has a list arr and a dics dict to access objects of the set"""
    def __init__(self):
        DictObjectManager_With_Id.__init__(self)

    ## Uses dict because is faster
    def values_order_by_date(self):
        return sorted(self.dic.values(), key=lambda o: o.date)

class DictObjectManager_With_IdDatetime(DictObjectManager_With_Id):
    """Base clase to create Sets, it needs id and name attributes, as index. It has a list arr and a dics dict to access objects of the set"""

    def __init__(self):
        DictObjectManager_With_Id.__init__(self)

    ## Uses dict because is faster
    def values_order_by_datetime(self):
        return sorted(self.dic.values(), key=lambda o: o.datetime)

## Usefull when creating a class with two attributes self.id and self.name only
class Object_With_IdName:
    ## Constructor with the following attributes combination
    ## 1. Object_With_IdName(). Create an Object_With_IdName with all attributes to None
    ## 2. Object_With_IdName( id,  name). Create an Object_With_IdName settings all attributes.
    ## @param name String with the name of the Object_With_IdName
    ## @param id Integer that sets the id of the Object_With_IdName
    def __init__(self, *args):
        def init__create( id,  name):
            self.id=id
            self.name=name
        if len(args)==0:
            init__create(None, None)
        if len(args)==2:
            init__create(*args)


class DictObjectManager_With_Id_Selectable(DictObjectManager_With_Id, ManagerSelection):
    def __init__(self):
        DictObjectManager_With_Id.__init__(self)
        ManagerSelection.__init__(self)

class DictObjectManager_With_IdDate_Selectable(DictObjectManager_With_IdDate, ManagerSelection):
    def __init__(self):
        DictObjectManager_With_IdDate.__init__(self)
        ManagerSelection.__init__(self)

class DictObjectManager_With_IdDatetime_Selectable(DictObjectManager_With_IdDatetime, ManagerSelection):
    def __init__(self):
        DictObjectManager_With_IdDatetime.__init__(self)
        ManagerSelection.__init__(self)

class DictObjectManager_With_IdName_Selectable(DictObjectManager_With_IdName, ManagerSelection):
    def __init__(self):
        DictObjectManager_With_IdName.__init__(self)
        ManagerSelection.__init__(self)

class ObjectManager_Selectable(ObjectManager, ManagerSelection):
    def __init__(self):
        ObjectManager.__init__(self)
        ManagerSelection.__init__(self)

class ObjectManager_With_Id_Selectable(ObjectManager_With_Id, ManagerSelection):
    def __init__(self):
        ObjectManager_With_Id.__init__(self)
        ManagerSelection.__init__(self)

class ObjectManager_With_IdDate_Selectable(ObjectManager_With_IdDate, ManagerSelection):
    def __init__(self):
        ObjectManager_With_IdDate.__init__(self)
        ManagerSelection.__init__(self)

class ObjectManager_With_IdDatetime_Selectable(ObjectManager_With_IdDatetime, ManagerSelection):
    def __init__(self):
        ObjectManager_With_IdDatetime.__init__(self)
        ManagerSelection.__init__(self)

class ObjectManager_With_IdName_Selectable(ObjectManager_With_IdName, ManagerSelection):
    def __init__(self):
        ObjectManager_With_IdName.__init__(self)
        ManagerSelection.__init__(self)


if __name__ == "__main__":
     import datetime
     sizes=(1,10,100,1000,10000,100000,1000000,3000000)
     for size in sizes:
         l=ObjectManager_With_Id()
         d=DictObjectManager_With_Id()
         for number in range(size):
             o=Object_With_IdName(number,"Name {}".format(number))
             l.append(o)
             d.append(o)
         middle=size*2//3
         start=datetime.datetime.now()
         l.find_by_id(middle)
         ltime=datetime.datetime.now()-start
         start=datetime.datetime.now()
         d.find_by_id(middle)
         dtime=datetime.datetime.now()-start
         print("Benchmarking search_by_id in to element {} with {} objects".format(middle,size))
         if ltime>=dtime:
             print("  * ObjectManager took {} more time than DictObjectManager".format(ltime-dtime))
         else:
             print("  * DictObjectManager took {} more time than ObjectManager".format(dtime-ltime))




# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'xulpymoney/ui/wdgDatetime.ui'
#
# Created by: PyQt5 UI code generator 5.12.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_wdgDatetime(object):
    def setupUi(self, wdgDatetime):
        wdgDatetime.setObjectName("wdgDatetime")
        wdgDatetime.resize(439, 254)
        self.horizontalLayout = QtWidgets.QHBoxLayout(wdgDatetime)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.grp = QtWidgets.QGroupBox(wdgDatetime)
        self.grp.setObjectName("grp")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.grp)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.teDate = QtWidgets.QCalendarWidget(self.grp)
        self.teDate.setFirstDayOfWeek(QtCore.Qt.Monday)
        self.teDate.setVerticalHeaderFormat(QtWidgets.QCalendarWidget.NoVerticalHeader)
        self.teDate.setObjectName("teDate")
        self.horizontalLayout_2.addWidget(self.teDate)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_2.addItem(spacerItem)
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.teTime = QtWidgets.QTimeEdit(self.grp)
        self.teTime.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.teTime.setObjectName("teTime")
        self.verticalLayout.addWidget(self.teTime)
        self.teMicroseconds = QtWidgets.QSpinBox(self.grp)
        self.teMicroseconds.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.teMicroseconds.setMaximum(999999)
        self.teMicroseconds.setObjectName("teMicroseconds")
        self.verticalLayout.addWidget(self.teMicroseconds)
        self.cmbZone = QtWidgets.QComboBox(self.grp)
        self.cmbZone.setObjectName("cmbZone")
        self.verticalLayout.addWidget(self.cmbZone)
        self.verticalLayout_2.addLayout(self.verticalLayout)
        self.line = QtWidgets.QFrame(self.grp)
        self.line.setFrameShape(QtWidgets.QFrame.HLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.verticalLayout_2.addWidget(self.line)
        self.cmdNow = QtWidgets.QPushButton(self.grp)
        self.cmdNow.setObjectName("cmdNow")
        self.verticalLayout_2.addWidget(self.cmdNow)
        spacerItem1 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_2.addItem(spacerItem1)
        self.horizontalLayout_2.addLayout(self.verticalLayout_2)
        self.verticalLayout_4.addLayout(self.horizontalLayout_2)
        self.horizontalLayout.addWidget(self.grp)

        self.retranslateUi(wdgDatetime)
        QtCore.QMetaObject.connectSlotsByName(wdgDatetime)

    def retranslateUi(self, wdgDatetime):
        _translate = QtCore.QCoreApplication.translate
        self.grp.setTitle(_translate("wdgDatetime", "Select a datetime"))
        self.teTime.setToolTip(_translate("wdgDatetime", "Select a time"))
        self.teTime.setDisplayFormat(_translate("wdgDatetime", "HH:mm:ss"))
        self.teMicroseconds.setToolTip(_translate("wdgDatetime", "Select microseconds"))
        self.cmdNow.setToolTip(_translate("wdgDatetime", "Set current time"))
        self.cmdNow.setText(_translate("wdgDatetime", "Now"))





import datetime
from PyQt5.QtCore import pyqtSlot
from PyQt5.QtWidgets import QMenu, QWidget, QTableWidgetItem, QDialog, QVBoxLayout
from xulpymoney.ui.Ui_wdgIndexRange import Ui_wdgIndexRange
from xulpymoney.libxulpymoney import Assets, Percentage
from xulpymoney.libxulpymoneyfunctions import qcenter
from xulpymoney.libxulpymoneytypes import eProductType,  eQColor
from xulpymoney.ui.frmProductReport import frmProductReport
from xulpymoney.ui.frmQuotesIBM import frmQuotesIBM
from xulpymoney.ui.wdgCalculator import wdgCalculator
from decimal import Decimal

class Range:
    def __init__(self, product,  bottom , top, price):
        """Calcula los rangos
        price, es el precio puede ser pasado el ´ultimo o el pen´ultimo"""
        self.product=product
        self.bottom=bottom
        self.top=top
        self.middle=Decimal((self.bottom+self.top)/2)
        self.current=price
        
    def currentPriceBottomVariation(self):
        """Calcs variation percentage from current price to bottom price"""
        return  Percentage(self.bottom-self.current,  self.current)
        
    def currentPriceTopVariation(self):
        return  Percentage(self.top-self.current, self.current)
    
    def currentPriceMiddleVariation(self):
        return  Percentage(self.middle-self.current, self.current)
        
    def textBottom(self):
        """Used to action text"""
        return ("Bottom: {0}. Variation: {1}".format(self.product.currency.string(self.bottom), self.currentPriceBottomVariation()))

    def textMiddle(self):
        """Used to action text"""
        return ("Middle: {0}. Variation: {1}".format(self.product.currency.string(self.middle), self.currentPriceMiddleVariation()))

    def textTop(self):
        """Used to action text"""
        return ("Top: {0}. Variation: {1}".format(self.product.currency.string(self.top), self.currentPriceTopVariation()))

class wdgIndexRange(QWidget, Ui_wdgIndexRange):
    def __init__(self,mem, parent=None):
        QWidget.__init__(self, parent)
        self.setupUi(self)
        self.mem=mem
         
        
        self.benchmark=self.mem.data.benchmark
        
        
        self.table.settings(self.mem, "wdgIndexRange")
        self.table.setVerticalHeaderHeight(None)#Must be after settings
                
        self.spin.setValue(float(self.mem.settingsdb.value("wdgIndexRange/spin", "2")))
        self.txtInvertir.setText(Decimal(self.mem.settingsdb.value("wdgIndexRange/invertir", "10000")))
        self.txtMinimo.setText(Decimal(self.mem.settingsdb.value("wdgIndexRange/minimo", "1000")))        
        
        self.cmbBenchmarkCurrent_load()
        self.load_data()
        
        self.selRange=None#Range() in right click
        
    def cmbBenchmarkCurrent_load(self):       
        if self.benchmark:
            self.cmbBenchmarkCurrent.clear() 
            self.cmbBenchmarkCurrent.addItem(self.tr("Benchmark penultimate price ({}) is {}".format(str(self.benchmark.result.basic.penultimate.datetime)[:16], self.benchmark.currency.string(self.benchmark.result.basic.penultimate.quote))))
            self.cmbBenchmarkCurrent.addItem(self.tr("Benchmark last price ({}) is {}. Last daily variation: {}.".format(str(self.benchmark.result.basic.last.datetime)[:16], self.benchmark.currency.string(self.benchmark.result.basic.last.quote), self.benchmark.result.basic.tpc_diario()) ))
            self.cmbBenchmarkCurrent.setCurrentIndex(1)#Last price
            
    def cmbBenchmarkCurrent_price(self):
        """Returns price of Benchmark selected in combo"""
        if self.cmbBenchmarkCurrent.currentIndex()==1:
            return self.benchmark.result.basic.last.quote
        else:
            return self.benchmark.result.basic.penultimate.quote

    def load_data(self):
        def inversiones(arr,min,max):
            resultado=""
            for i in arr:
                if i[0]>=min and i[0]<max:
                    o=i[1]
                    resultado=resultado+ self.tr("{0} {1} ({2}): {3} shares of {4} = {5}\n".format(str(o.datetime)[:16], o.investment.name, o.investment.account.name, round(o.shares, 0),  o.price(type=1), o.gross(type=1)))
            return resultado[:-1]
        ######################################################
        inicio=datetime.datetime.now()

        #Makes and array arr with investment current operations and sorts it
        arr=[]
        maxoper=0
        for i in self.mem.data.investments_active().arr:
                for o in i.op_actual.arr:
                    if self.cmbShowOptions.currentIndex()==0 and o.show_in_ranges==True:#Show qualified        
                        if maxoper<o.referenciaindice.quote:
                            maxoper=o.referenciaindice.quote
                        arr.append((o.referenciaindice.quote, o))                    
                    elif self.cmbShowOptions.currentIndex()==1 and i.product.type.id in (eProductType.Share, eProductType.Warrant, eProductType.ETF):            #Shares, Warrants, ETF
                        if maxoper<o.referenciaindice.quote:
                            maxoper=o.referenciaindice.quote
                        arr.append((o.referenciaindice.quote, o))             
                    elif self.cmbShowOptions.currentIndex()==2 and i.product.type.id in (eProductType.PrivateBond, eProductType.PublicBond):            #Bonds
                        if maxoper<o.referenciaindice.quote:
                            maxoper=o.referenciaindice.quote
                        arr.append((o.referenciaindice.quote, o))             
                    elif self.cmbShowOptions.currentIndex()==3 and i.product.type.id in (eProductType.Fund, ):            #Funds
                        if maxoper<o.referenciaindice.quote:
                            maxoper=o.referenciaindice.quote
                        arr.append((o.referenciaindice.quote, o))       
                    elif self.cmbShowOptions.currentIndex()==4:            
                        if maxoper<o.referenciaindice.quote:
                            maxoper=o.referenciaindice.quote
                        arr.append((o.referenciaindice.quote, o))             
        arr=sorted(arr, key=lambda row: row[1].datetime,  reverse=False) 
        
        #Makes and array from  minimum to benchmark maximum + 2% to minimum
        ranges=[]       
        if maxoper==0:##NO hay operinvestments
            maximo=self.cmbBenchmarkCurrent_price()*(1+2*Decimal(self.spin.value()/100))
        else:
            maximo=maxoper*(1+2*Decimal(self.spin.value()/100))##1.04 en caso de 2
        minimo=self.txtMinimo.decimal()
        PuntRange=minimo
        while PuntRange<maximo:
            ranges.insert(0, PuntRange)
            PuntRange=PuntRange*(1+Decimal(self.spin.value()/100))
    
        #Calculate zero risk assests and range number covered
        zeroriskplusbonds=Assets(self.mem).patrimonio_riesgo_cero(self.mem.data.investments_active(), datetime.date.today()).amount# +Assets(self.mem).saldo_todas_inversiones_bonds(datetime.date.today()).amount
        rangescovered=int(zeroriskplusbonds/self.txtInvertir.decimal())
        
        #Iterates all ranges and prints table
        self.table.applySettings()
        self.table.clearContents()
        self.table.setRowCount(len(ranges))
        colorized=0
        for i, r in enumerate(ranges):###De mayor a menor
            top=r*(1+Decimal(self.spin.value()/100))
            bottom=r
            self.table.setItem(i, 0,qcenter("{}-{}".format(int(bottom), int(top))))
            self.table.setItem(i, 1,QTableWidgetItem(inversiones(arr, bottom, top)))
            if bottom<self.cmbBenchmarkCurrent_price():
                if self.cmbBenchmarkCurrent_price()<=top: ##Colorize current price
                    self.table.item(i, 0).setBackground(eQColor.Red)
                if colorized<=rangescovered:
                    self.table.item(i, 1).setBackground(eQColor.Green)
                    colorized=colorized+1

        #Prints label
        self.lblTotal.setText(self.tr("{} green colorized ranges of {} benchmark are covered by zero risk and bonds balance ({}).").format(colorized, self.benchmark.name, self.mem.localcurrency.string(zeroriskplusbonds)))
        print ("wdgIndexRange > load_data: {0}".format(datetime.datetime.now()-inicio))

    def on_cmd_pressed(self):
        self.mem.settingsdb.setValue("wdgIndexRange/spin", self.spin.value())
        self.mem.settingsdb.setValue("wdgIndexRange/invertir", self.txtInvertir.text())
        self.mem.settingsdb.setValue("wdgIndexRange/minimo", self.txtMinimo.text())
        self.load_data()

    @pyqtSlot(int)  
    def on_cmbBenchmarkCurrent_currentIndexChanged(self, index):
        self.load_data()
        
    def on_cmbShowOptions_currentIndexChanged(self, index):
        self.load_data()

    def on_cmdIRAnalisis_pressed(self):
        w=frmProductReport(self.mem, self.benchmark, None,  self)
        w.exec_()
        
    def on_cmdIRInsertar_pressed(self):
        w=frmQuotesIBM(self.mem, self.benchmark, None,  self)
        w.exec_() 
        self.benchmark.needStatus(2, downgrade_to=0)
        self.cmbBenchmarkCurrent_load()
        self.load_data()

    def on_table_customContextMenuRequested(self,  pos):
        if self.range!=None:
            self.actionBottom.setText(self.range.textBottom())
            self.actionTop.setText(self.range.textTop())
            self.actionMiddle.setText(self.range.textMiddle())
            menu=QMenu()
            menu.addAction(self.actionTop)
            menu.addAction(self.actionMiddle)   
            menu.addAction(self.actionBottom)
            menu.exec_(self.table.mapToGlobal(pos))

    def on_table_itemSelectionChanged(self):
        try:
            for i in self.table.selectedItems():#itera por cada item no row.
                if i.column()==0:
                    self.range=Range(self.benchmark, int(i.text().split("-")[0]), int(i.text().split("-")[1]), self.cmbBenchmarkCurrent_price())
        except:
            self.range=None

    @pyqtSlot() 
    def on_actionBottom_triggered(self):        
        d=QDialog(self)        
        d.setFixedSize(850, 850)
        d.setWindowTitle(self.tr("Investment calculator"))
        w=wdgCalculator(self.mem)
        w.setProduct(self.mem.data.products.find_by_id(int(self.mem.settings.value("wdgCalculator/product", "0"))))
        w.spnProductPriceVariation.setValue(self.range.currentPriceBottomVariation().value_100())
        lay = QVBoxLayout(d)
        lay.addWidget(w)
        if w.hasProducts==True:
            d.show()
        else:
            d.close()

    @pyqtSlot() 
    def on_actionTop_triggered(self):        
        d=QDialog(self)
        d.setFixedSize(850, 850)
        d.setWindowTitle(self.tr("Investment calculator"))
        w=wdgCalculator(self.mem)
        w.setProduct(self.mem.data.products.find_by_id(int(self.mem.settings.value("wdgCalculator/product", "0"))))
        w.spnProductPriceVariation.setValue(self.range.currentPriceTopVariation().value_100())
        lay = QVBoxLayout(d)
        lay.addWidget(w)
        if w.hasProducts==True:
            d.show()
        else:
            d.close()

    @pyqtSlot() 
    def on_actionMiddle_triggered(self):        
        d=QDialog(self)        
        d.setFixedSize(850, 850)
        d.setWindowTitle(self.tr("Investment calculator"))
        w=wdgCalculator(self.mem)
        w.setProduct(self.mem.data.products.find_by_id(int(self.mem.settings.value("wdgCalculator/product", "0"))))
        w.spnProductPriceVariation.setValue(self.range.currentPriceMiddleVariation().value_100())
        lay = QVBoxLayout(d)
        lay.addWidget(w)
        if w.hasProducts==True:
            d.show()
        else:
            d.close()
        



from PyQt5.QtCore import pyqtSignal, pyqtSlot
from PyQt5.QtWidgets import QWidget
import datetime
from xulpymoney.ui.Ui_wdgYearMonth import Ui_wdgYearMonth
from xulpymoney.libxulpymoneyfunctions import function_name, qmessagebox

class wdgYearMonth(QWidget, Ui_wdgYearMonth):
    changed=pyqtSignal()
    def __init__(self,  parent = None, name = None):
        QWidget.__init__(self,  parent)
        self.setupUi(self)
        
        
    def initiate(self, firstyear,  lastyear, currentyear, currentmonth):
        """Debe ser la primera función después del constructor"""
        if firstyear==None:
            self.setEnabled(False)
            print (function_name(self), "Firstyear is None")
            return
        
        self.firstyear=firstyear
        self.lastyear=lastyear
        self.year=currentyear
        self.month=currentmonth
        for year in range(firstyear, lastyear+1):
            self.cmbYear.addItem(str(year), year)
        self.set(currentyear, currentmonth)
        
    def set(self,  year , month):
        self.year=year
        self.month=month
        self.cmbYear.setCurrentIndex(self.year-self.firstyear)
        self.cmbMonth.setCurrentIndex(self.month-1)

    @pyqtSlot(str)      
    def on_cmbYear_currentIndexChanged(self, text):
        self.year=int(text)
        self.changed.emit()
        
    @pyqtSlot(int)      
    def on_cmbMonth_currentIndexChanged(self, integ):
        self.month=integ+1
        self.changed.emit()
        
    def on_cmdNext_pressed(self):
        if self.month==12:
            if self.year==self.lastyear:
                qmessagebox(self.tr("I can't show the next month"))
                return
            self.month=1
            self.year=self.year+1
        else:
            self.month=self.month+1
        self.set(self.year, self.month)
        
    def on_cmdPrevious_pressed(self):
        if self.month==1:
            if self.firstyear==self.year:
                qmessagebox(self.tr("I can't show the previous month"))
                return
            self.month=12
            self.year=self.year-1
        else:
            self.month=self.month-1
        self.set(self.year, self.month)
        
    @pyqtSlot()      
    def on_cmdCurrent_pressed(self):
        self.set(datetime.date.today().year, datetime.date.today().month)
        
        




from PyQt5.QtCore import pyqtSlot, Qt
from PyQt5.QtWidgets import QWidget, QDialog, QVBoxLayout, QMenu, QMessageBox
from xulpymoney.ui.Ui_wdgProducts import Ui_wdgProducts
from xulpymoney.ui.frmProductReport import frmProductReport
from xulpymoney.libmanagers import ManagerSelectionMode
from xulpymoney.libxulpymoney import QuoteAllIntradayManager
from xulpymoney.libxulpymoneyfunctions import qmessagebox
from xulpymoney.ui.frmQuotesIBM import frmQuotesIBM
from xulpymoney.ui.wdgProductsDataMove import wdgProductsDataMove
from xulpymoney.ui.frmEstimationsAdd import frmEstimationsAdd
from xulpymoney.ui.wdgProductHistoricalChart import wdgProductHistoricalBuyChart
import logging

class wdgProducts(QWidget, Ui_wdgProducts):
    def __init__(self, mem,  arrInt=[],  parent=None):
        QWidget.__init__(self, parent)
        self.setupUi(self)
        self.mem=mem
        self.tblInvestments.settings(self.mem, "wdgProducts")
        self.mem.stockmarkets.qcombobox(self.cmbStockExchange)
        self.arrInt=arrInt#Lista de ids of products showed and used to show
        self.build_array_from_arrInt()

    def build_array_from_arrInt(self):        
        self.products=self.mem.data.products.ProductManager_with_id_in_list(self.arrInt)
        self.products.setSelectionMode(ManagerSelectionMode.List)
        self.products.needStatus(needstatus=1, progress=True)
        self.products.order_by_upper_name()
        self.lblFound.setText(self.tr("Found {0} records".format(self.products.length())))
        self.products.myqtablewidget(self.tblInvestments)

    @pyqtSlot()
    def on_actionFavorites_triggered(self):      
        if self.products.selected[0].id in self.mem.favorites:
            self.mem.favorites.remove(self.products.selected[0].id)
        else:
            self.mem.favorites.append(self.products.selected[0].id)
        logging.debug("Favoritos: {}".format(self.mem.favorites))
        self.mem.save_MemSettingsDB()
        
        del self.arrInt
        self.arrInt=[]
        for f in self.mem.favorites:
            self.arrInt.append(f)
        self.build_array_from_arrInt()
        

    @pyqtSlot()  
    def on_actionIbex35_triggered(self):
        del self.arrInt
        self.arrInt=[]
        for p in self.mem.data.products.arr:
            if p.agrupations.dbstring.find("|IBEX|")!=-1 and p.obsolete==False:
                self.arrInt.append(p.id)
        self.build_array_from_arrInt()

    @pyqtSlot() 
    def on_actionProductDelete_triggered(self):
        if self.products.selected[0].is_deletable()==False:
            qmessagebox(self.tr("This product can't be removed, because is marked as not remavable"))
            return
            
        if self.products.selected[0].is_system()==True:
            qmessagebox(self.tr("This product can't be removed, because is a system product"))
            return
            
        respuesta = QMessageBox.warning(self, self.tr("Xulpymoney"), self.tr("Deleting data from selected product ({0}). If you use manual update mode, data won't be recovered. Do you want to continue?".format(self.products.selected[0].id)), QMessageBox.Ok | QMessageBox.Cancel)
        if respuesta==QMessageBox.Ok:
            self.arrInt.remove(self.products.selected[0].id)
            self.mem.data.products.remove(self.products.selected[0])
            self.mem.con.commit()
            self.build_array_from_arrInt()            

    @pyqtSlot() 
    def on_actionProductNew_triggered(self):
        w=frmProductReport(self.mem, None, self)
        w.exec_()        
        del self.arrInt
        self.arrInt=[w.product.id, ]
        self.build_array_from_arrInt()

    @pyqtSlot() 
    def on_actionPurchaseGraphic_triggered(self):
        self.products.selected[0].needStatus(2)
        d=QDialog(self)     
        d.showMaximized()
        d.setWindowTitle(self.tr("Purchase graph"))
        lay = QVBoxLayout(d)
        
        wc=wdgProductHistoricalBuyChart()
        wc.setProduct(self.products.selected[0], None)
        wc.setPrice(self.products.selected[0].result.basic.last.quote)
        wc.generate()
        wc.display()
        lay.addWidget(wc)
        d.exec_()
        
    @pyqtSlot() 
    def on_actionProductReport_triggered(self):
        w=frmProductReport(self.mem, self.products.selected[0], None,  self)
        w.exec_()        
        self.build_array_from_arrInt()
        
    @pyqtSlot() 
    def on_actionSortTPCDiario_triggered(self):
        if self.products.order_by_daily_tpc():
            self.products.myqtablewidget(self.tblInvestments)        
        else:
            qmessagebox(self.tr("I couldn't order data due to they have null values"))

    @pyqtSlot()
    def on_actionSortTPCAnual_triggered(self):
        if self.products.order_by_annual_tpc():
            self.products.myqtablewidget(self.tblInvestments)        
        else:
            qmessagebox(self.tr("I couldn't order data due to they have null values"))

    @pyqtSlot()
    def on_actionSortHour_triggered(self):
        self.products.order_by_datetime()
        self.products.myqtablewidget(self.tblInvestments)        

    @pyqtSlot()
    def on_actionSortName_triggered(self):
        self.products.order_by_upper_name()
        self.products.myqtablewidget(self.tblInvestments)        

    @pyqtSlot()
    def on_actionSortDividend_triggered(self):
        if self.products.order_by_dividend():
            self.products.myqtablewidget(self.tblInvestments)        
        else:
            qmessagebox(self.tr("I couldn't order data due to they have null values"))     
        
    def on_txt_returnPressed(self):
        self.on_cmd_pressed()

    def on_tblInvestments_cellDoubleClicked(self, row, column):
        self.on_actionProductReport_triggered()

    def on_cmd_pressed(self):
        if len(self.txt.text().upper())<=2:            
            qmessagebox(self.tr("Search too wide. You need more than 2 characters"))
            return

        # To filter by stockmarket
        sm=None
        if self.chkStockExchange.checkState()==Qt.Checked:
            sm=self.mem.stockmarkets.find_by_id(self.cmbStockExchange.itemData(self.cmbStockExchange.currentIndex()))     

        del self.arrInt
        self.arrInt=[]
        #Temporal ProductManager
        pros=self.mem.data.products.ProductManager_contains_string(self.txt.text())
        for p in pros.arr:
            #Filter sm
            if sm!=None and sm.id!=p.stockmarket.id:
                continue
            else:
                self.arrInt.append(p.id)

        self.build_array_from_arrInt()

    def on_tblInvestments_customContextMenuRequested(self,  pos):
        menu=QMenu()
        menu.addAction(self.actionProductReport)
        menu.addAction(self.actionPurchaseGraphic)
        menu.addSeparator()
        menu.addAction(self.actionProductNew)
        menu.addAction(self.actionProductDelete)
        menu.addSeparator()
        menu.addAction(self.actionQuoteNew)
        menu.addAction(self.actionProductPriceLastRemove)
        menu.addAction(self.actionEstimationDPSNew)
        menu.addSeparator()
        menu.addAction(self.actionMergeCodes)
        menu.addAction(self.actionFavorites)
        if len(self.products.selected)==1:
            if self.products.selected[0].id in self.mem.favorites:
                self.actionFavorites.setText(self.tr("Remove from favorites"))
            else:
                self.actionFavorites.setText(self.tr("Add to favorites"))
        menu.addSeparator()
        menu.addAction(self.actionPurge)

        if len (self.products.selected)==1:
            if self.products.selected[0].id==79329:
                menu.addSeparator()
                menu.addAction(self.actionIbex35)
        menu.addSeparator()
        ordenar=QMenu(self.tr("Order by"))
        menu.addMenu(ordenar)
        ordenar.addAction(self.actionSortName)
        ordenar.addAction(self.actionSortHour)
        ordenar.addAction(self.actionSortTPCDiario)
        ordenar.addAction(self.actionSortTPCAnual)
        ordenar.addAction(self.actionSortDividend)
        
        #Enabled disabled  
        if len(self.products.selected)==1:
            self.actionMergeCodes.setEnabled(False)
            self.actionProductDelete.setEnabled(True)
            self.actionFavorites.setEnabled(True)
            self.actionProductReport.setEnabled(True)
            self.actionPurchaseGraphic.setEnabled(True)
            self.actionIbex35.setEnabled(True)
            self.actionQuoteNew.setEnabled(True)
            self.actionEstimationDPSNew.setEnabled(True)
            self.actionPurge.setEnabled(True)
            self.actionProductPriceLastRemove.setEnabled(True)
        elif len(self.products.selected)==2:
            self.actionMergeCodes.setEnabled(True)
        else:
            self.actionMergeCodes.setEnabled(False)
            self.actionProductDelete.setEnabled(False)
            self.actionFavorites.setEnabled(False)
            self.actionProductReport.setEnabled(False)
            self.actionPurchaseGraphic.setEnabled(False)
            self.actionIbex35.setEnabled(False)
            self.actionQuoteNew.setEnabled(False)
            self.actionEstimationDPSNew.setEnabled(False)
            self.actionPurge.setEnabled(False)
            self.actionProductPriceLastRemove.setEnabled(False)
        menu.exec_(self.tblInvestments.mapToGlobal(pos))

    @pyqtSlot() 
    def on_actionMergeCodes_triggered(self):
        #Only two checked in custom contest
        d=QDialog(self)
        d.setWindowTitle(self.tr("Merging codes"))
        w=wdgProductsDataMove(self.mem, self.products.selected[0], self.products.selected[1])
        lay = QVBoxLayout(d)
        lay.addWidget(w)
        d.resize(w.size())
        d.exec_()
        self.build_array_from_arrInt()
    
    def on_tblInvestments_itemSelectionChanged(self):
        self.products.cleanSelection()
        for i in self.tblInvestments.selectedItems():
            if i.column()==0:#only once per row
                self.products.selected.append(self.products.arr[i.row()])
        logging.debug(self.products.selected)

    @pyqtSlot()  
    def on_actionPurge_triggered(self):
        all=QuoteAllIntradayManager(self.mem)
        all.load_from_db(self.products.selected[0])
        numpurged=all.purge(progress=True)
        if numpurged!=None:#Canceled
            self.mem.con.commit()
            qmessagebox(self.tr("{0} quotes have been purged from {1}".format(numpurged, self.products.selected[0].name)))
        else:
            self.mem.con.rollback()

    @pyqtSlot()  
    def on_actionQuoteNew_triggered(self):
        w=frmQuotesIBM(self.mem,  self.products.selected[0])
        w.exec_()
        self.build_array_from_arrInt()

    @pyqtSlot() 
    def on_actionProductPriceLastRemove_triggered(self):
        self.products.selected[0].result.basic.last.delete()
        self.mem.con.commit()
        self.products.selected[0].needStatus(1, downgrade_to=0)
        self.build_array_from_arrInt()

    @pyqtSlot()  
    def on_actionEstimationDPSNew_triggered(self):
        d=frmEstimationsAdd(self.mem, self.products.selected[0], "dps")
        d.exec_()
        if d.result()==QDialog.Accepted:
            self.products.selected[0].needStatus(1, downgrade_to=0)
            self.build_array_from_arrInt()



import pkg_resources
from PyQt5.QtCore import QTranslator,  pyqtSlot, QLocale
from PyQt5.QtWidgets import QDialog, QMessageBox, QApplication
from xulpymoney.ui.Ui_frmInit import Ui_frmInit
from xulpymoney.libxulpymoney import MemXulpymoney
from xulpymoney.admin_pg import AdminPG
from xulpymoney.libxulpymoneyfunctions import qmessagebox
from xulpymoney.libxulpymoneytypes import eOperationType

class frmInit(QDialog, Ui_frmInit):
    def __init__(self, parent = None, name = None, modal = False):
        QDialog.__init__(self,  parent)
        locale=QLocale()
        id=locale.system().name()
        if len(id)!=2:
            id=id[:-len(id)+2]
        print("Locale {} detected".format(id))
        self.setupUi(self)
        self.mem=MemXulpymoney()      
        self.mem.setQTranslator(QTranslator(QApplication.instance()))  
        self.mem.languages.qcombobox(self.cmbLanguage, self.mem.languages.find_by_id(id))
    
    @pyqtSlot(str)      
    def on_cmbLanguage_currentIndexChanged(self, stri):
        self.mem.language=self.mem.languages.find_by_id(self.cmbLanguage.itemData(self.cmbLanguage.currentIndex()))
        self.mem.languages.cambiar(self.mem.language.id)
        self.retranslateUi(self)
    
    @pyqtSlot()
    def on_cmdCreate_released(self):
        respuesta = QMessageBox.warning(self, self.windowTitle(), self.tr("Do you want to create {} database in {}?".format(self.txtXulpymoney.text(), self.cmbLanguage.currentText())), QMessageBox.Ok | QMessageBox.Cancel)
        if respuesta==QMessageBox.Ok:             
            self.cmbLanguage.setEnabled(False)
            self.txtPass.setEnabled(False)
            self.txtPort.setEnabled(False)
            self.txtServer.setEnabled(False)
            self.txtUser.setEnabled(False)
            self.txtXulpymoney.setEnabled(False)
   
            self.admin=AdminPG(self.txtUser.text(), self.txtPass.text(),  self.txtServer.text(),  self.txtPort.text())
                        
            if self.admin.con.is_active()==False:
                qmessagebox(self.tr("Error conecting to table template1 in database server"))
                self.reject()
                return            

            if self.admin.create_db(self.txtXulpymoney.text())==False:
                qmessagebox(self.tr("Error creating database. You need to be superuser or maybe it already exist"))
                self.reject()
                return
            
            #Una vez creada la base de datos me conecto
            self.newdbcon=self.admin.connect_to_database(self.txtXulpymoney.text())
            
            if self.xulpymoney_basic_schema()==False:
                qmessagebox(self.tr("Error processing SQL init scripts"))
                self.reject()
                return
            self.newdbcon.commit()
            self.cmdCreate.setEnabled(False)

            qmessagebox(self.tr("Database created. User xulpymoney_user and xulpymoney_admin have been created. Please run Xulpymoney and login"))        
            print ("App correctly closed")
            self.close()
        else:
            self.cmbLanguage.setEnabled(True)
            self.txtPass.setEnabled(True)
            self.txtPort.setEnabled(True)
            self.txtServer.setEnabled(True)
            self.txtUser.setEnabled(True)
            self.txtXulpymoney.setEnabled(True)

    ## In xulpymoney.sql there aren't concepts so we must add them after running it
    ## This strings are tranlatted in hardcoded_strings module
    def xulpymoney_basic_schema(self):
            filename=pkg_resources.resource_filename("xulpymoney","sql/xulpymoney.sql")
            self.newdbcon.load_script(filename)
            
            cur= self.newdbcon.cursor()
            cur.execute("insert into public.entidadesbancarias values(3,'{0}', true)".format("Personal Management"))
            cur.execute("insert into public.cuentas values(4,'{0}',3,true,NULL,'EUR')".format("Cash"))
            cur.execute("insert into public.conceptos values(1,'{0}',2,false)".format("Initiating bank account"))
            cur.execute("insert into public.conceptos values(2,'{0}',2,true)".format("Paysheet"))
            cur.execute("insert into public.conceptos values(3,'{0}',1,true)".format("Supermarket"))
            cur.execute("insert into public.conceptos values(4,'{0}',3,false)".format("Transfer. Origin"))
            cur.execute("insert into public.conceptos values(5,'{0}',3,false)".format("Transfer. Destination"))
            cur.execute("insert into public.conceptos values(6,'{0}',2,false)".format("Taxes. Returned"))
            cur.execute("insert into public.conceptos values(7,'{0}',1,true)".format("Gas"))
            cur.execute("insert into public.conceptos values(8,'{0}',1,true)".format("Restaurant"))  
            cur.execute("insert into public.conceptos values(29,'{0}',4,false)".format("Purchase investment product"))
            cur.execute("insert into public.conceptos values(35,'{0}',5,false)".format("Sale investment product"))
            cur.execute("insert into public.conceptos values(37,'{0}',1,false)".format("Taxes. Paid"))
            cur.execute("insert into public.conceptos values(38,'{0}',1,false)".format("Bank commissions"))
            cur.execute("insert into public.conceptos values(39,'{0}',2,false)".format("Dividends"))
            cur.execute("insert into public.conceptos values(40,'{0}',7,false)".format("Credit card billing"))
            cur.execute("insert into public.conceptos values(43,'{0}',6,false)".format("Added shares"))
            cur.execute("insert into public.conceptos values(50,'{0}',2,false)".format("Attendance bonus"))
            cur.execute("insert into public.conceptos values(59,'{0}',1,false)".format("Custody commission"))
            cur.execute("insert into public.conceptos values(62,'{0}',2,false)".format("Dividends. Sale of rights"))
            cur.execute("insert into public.conceptos values(63,'{0}',1,false)".format("Bonds. Running coupon payment"))
            cur.execute("insert into public.conceptos values(65,'{0}',2,false)".format("Bonds. Running coupon collection"))
            cur.execute("insert into public.conceptos values(66,'{0}',2,false)".format("Bonds. Coupon collection"))
            cur.execute("insert into public.conceptos values(67,'{0}',2,false)".format("Credit card refund"))          
            cur.execute("insert into public.conceptos values(68,%s,%s,false)",("HL adjustment income", eOperationType.Income))    
            cur.execute("insert into public.conceptos values(69,%s,%s,false)",("HL adjustment expense", eOperationType.Expense))  
            cur.execute("insert into public.conceptos values(70,%s,%s,false)",("HL Guarantee payment", eOperationType.Expense))  
            cur.execute("insert into public.conceptos values(71,%s,%s,false)",("HL Guarantee return", eOperationType.Income))     
            cur.execute("insert into public.conceptos values(72,%s,%s,false)",("HL Operation commission", eOperationType.Expense))     
            cur.execute("insert into public.conceptos values(73,%s,%s,false)",("HL Paid interest", eOperationType.Expense))     
            cur.execute("insert into public.conceptos values(74,%s,%s,false)",("HL Received interest", eOperationType.Income))   
            cur.execute("insert into public.conceptos values(75,%s,%s,false)",("Rollover Paid", eOperationType.Expense))   
            cur.execute("insert into public.conceptos values(76,%s,%s,false)",("Rollover Received", eOperationType.Income))   
            cur.close()
#            return True
#        except:
#            print ("Error creating xulpymoney basic schema")
#            return False




# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'xulpymoney/ui/wdgInvestmentsOperations.ui'
#
# Created by: PyQt5 UI code generator 5.12.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_wdgInvestmentsOperations(object):
    def setupUi(self, wdgInvestmentsOperations):
        wdgInvestmentsOperations.setObjectName("wdgInvestmentsOperations")
        wdgInvestmentsOperations.resize(1048, 635)
        self.horizontalLayout = QtWidgets.QHBoxLayout(wdgInvestmentsOperations)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.lbl = QtWidgets.QLabel(wdgInvestmentsOperations)
        font = QtGui.QFont()
        font.setPointSize(14)
        font.setBold(True)
        font.setWeight(75)
        self.lbl.setFont(font)
        self.lbl.setAlignment(QtCore.Qt.AlignCenter)
        self.lbl.setObjectName("lbl")
        self.verticalLayout.addWidget(self.lbl)
        self.tab = QtWidgets.QTabWidget(wdgInvestmentsOperations)
        self.tab.setObjectName("tab")
        self.tab_1 = QtWidgets.QWidget()
        self.tab_1.setObjectName("tab_1")
        self.horizontalLayout_7 = QtWidgets.QHBoxLayout(self.tab_1)
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout()
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.groupBox = QtWidgets.QGroupBox(self.tab_1)
        self.groupBox.setObjectName("groupBox")
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout(self.groupBox)
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout()
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.radYear = QtWidgets.QRadioButton(self.groupBox)
        self.radYear.setObjectName("radYear")
        self.horizontalLayout_3.addWidget(self.radYear)
        self.wy = wdgYear(self.groupBox)
        self.wy.setObjectName("wy")
        self.horizontalLayout_3.addWidget(self.wy)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem)
        self.verticalLayout_3.addLayout(self.horizontalLayout_3)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.radYearMonth = QtWidgets.QRadioButton(self.groupBox)
        self.radYearMonth.setChecked(True)
        self.radYearMonth.setObjectName("radYearMonth")
        self.horizontalLayout_2.addWidget(self.radYearMonth)
        self.wym = wdgYearMonth(self.groupBox)
        self.wym.setObjectName("wym")
        self.horizontalLayout_2.addWidget(self.wym)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem1)
        self.verticalLayout_3.addLayout(self.horizontalLayout_2)
        self.verticalLayout_2.addLayout(self.verticalLayout_3)
        self.line = QtWidgets.QFrame(self.groupBox)
        self.line.setFrameShape(QtWidgets.QFrame.HLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.verticalLayout_2.addWidget(self.line)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(spacerItem2)
        self.cmbFilters = QtWidgets.QComboBox(self.groupBox)
        self.cmbFilters.setSizeAdjustPolicy(QtWidgets.QComboBox.AdjustToContents)
        self.cmbFilters.setObjectName("cmbFilters")
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/xulpymoney/star.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.cmbFilters.addItem(icon, "")
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/xulpymoney/add.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.cmbFilters.addItem(icon1, "")
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/xulpymoney/list-remove.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.cmbFilters.addItem(icon2, "")
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(":/xulpymoney/eye_red.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.cmbFilters.addItem(icon3, "")
        self.horizontalLayout_4.addWidget(self.cmbFilters)
        self.verticalLayout_2.addLayout(self.horizontalLayout_4)
        self.horizontalLayout_6.addLayout(self.verticalLayout_2)
        self.verticalLayout_4.addWidget(self.groupBox)
        self.table = myQTableWidget(self.tab_1)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.table.sizePolicy().hasHeightForWidth())
        self.table.setSizePolicy(sizePolicy)
        self.table.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.table.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.table.setAlternatingRowColors(True)
        self.table.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.table.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.table.setObjectName("table")
        self.table.setColumnCount(0)
        self.table.setRowCount(0)
        self.table.verticalHeader().setVisible(False)
        self.verticalLayout_4.addWidget(self.table)
        self.horizontalLayout_7.addLayout(self.verticalLayout_4)
        icon4 = QtGui.QIcon()
        icon4.addPixmap(QtGui.QPixmap(":/xulpymoney/document-edit.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.tab.addTab(self.tab_1, icon4, "")
        self.tab_2 = QtWidgets.QWidget()
        self.tab_2.setObjectName("tab_2")
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout(self.tab_2)
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.tblCurrent = myQTableWidget(self.tab_2)
        self.tblCurrent.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.tblCurrent.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.tblCurrent.setAlternatingRowColors(True)
        self.tblCurrent.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.tblCurrent.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.tblCurrent.setObjectName("tblCurrent")
        self.tblCurrent.setColumnCount(0)
        self.tblCurrent.setRowCount(0)
        self.tblCurrent.verticalHeader().setVisible(False)
        self.horizontalLayout_5.addWidget(self.tblCurrent)
        self.tab.addTab(self.tab_2, icon4, "")
        self.verticalLayout.addWidget(self.tab)
        self.horizontalLayout.addLayout(self.verticalLayout)
        self.actionShowInvestment = QtWidgets.QAction(wdgInvestmentsOperations)
        icon5 = QtGui.QIcon()
        icon5.addPixmap(QtGui.QPixmap(":/xulpymoney/bundle.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionShowInvestment.setIcon(icon5)
        self.actionShowInvestment.setObjectName("actionShowInvestment")
        self.actionShowInvestmentOperation = QtWidgets.QAction(wdgInvestmentsOperations)
        self.actionShowInvestmentOperation.setIcon(icon4)
        self.actionShowInvestmentOperation.setObjectName("actionShowInvestmentOperation")
        self.actionShowAccount = QtWidgets.QAction(wdgInvestmentsOperations)
        icon6 = QtGui.QIcon()
        icon6.addPixmap(QtGui.QPixmap(":/xulpymoney/coins.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionShowAccount.setIcon(icon6)
        self.actionShowAccount.setObjectName("actionShowAccount")
        self.actionShowProduct = QtWidgets.QAction(wdgInvestmentsOperations)
        icon7 = QtGui.QIcon()
        icon7.addPixmap(QtGui.QPixmap(":/xulpymoney/books.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionShowProduct.setIcon(icon7)
        self.actionShowProduct.setObjectName("actionShowProduct")
        self.actionRangeReport = QtWidgets.QAction(wdgInvestmentsOperations)
        icon8 = QtGui.QIcon()
        icon8.addPixmap(QtGui.QPixmap(":/xulpymoney/eye.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionRangeReport.setIcon(icon8)
        self.actionRangeReport.setObjectName("actionRangeReport")

        self.retranslateUi(wdgInvestmentsOperations)
        self.tab.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(wdgInvestmentsOperations)

    def retranslateUi(self, wdgInvestmentsOperations):
        _translate = QtCore.QCoreApplication.translate
        self.lbl.setText(_translate("wdgInvestmentsOperations", "Investment operations list"))
        self.groupBox.setTitle(_translate("wdgInvestmentsOperations", "List options"))
        self.radYear.setText(_translate("wdgInvestmentsOperations", "Show all investment operations from selected year"))
        self.radYearMonth.setText(_translate("wdgInvestmentsOperations", "Show investment operations from selected year and month"))
        self.cmbFilters.setItemText(0, _translate("wdgInvestmentsOperations", "Show all investment operations"))
        self.cmbFilters.setItemText(1, _translate("wdgInvestmentsOperations", "Show purchasing investment operations"))
        self.cmbFilters.setItemText(2, _translate("wdgInvestmentsOperations", "Show selling investment operations"))
        self.cmbFilters.setItemText(3, _translate("wdgInvestmentsOperations", "Exclude selling and purchasing investment operations"))
        self.tab.setTabText(self.tab.indexOf(self.tab_1), _translate("wdgInvestmentsOperations", "Operations list"))
        self.tab.setTabText(self.tab.indexOf(self.tab_2), _translate("wdgInvestmentsOperations", "Current investment status"))
        self.actionShowInvestment.setText(_translate("wdgInvestmentsOperations", "Show investment"))
        self.actionShowInvestment.setToolTip(_translate("wdgInvestmentsOperations", "Show investment"))
        self.actionShowInvestmentOperation.setText(_translate("wdgInvestmentsOperations", "Show investment operation"))
        self.actionShowInvestmentOperation.setToolTip(_translate("wdgInvestmentsOperations", "Show investment operation"))
        self.actionShowAccount.setText(_translate("wdgInvestmentsOperations", "Show account"))
        self.actionShowAccount.setToolTip(_translate("wdgInvestmentsOperations", "Show account"))
        self.actionShowProduct.setText(_translate("wdgInvestmentsOperations", "Show product"))
        self.actionShowProduct.setToolTip(_translate("wdgInvestmentsOperations", "Show product"))
        self.actionRangeReport.setText(_translate("wdgInvestmentsOperations", "Hide in range report"))
        self.actionRangeReport.setToolTip(_translate("wdgInvestmentsOperations", "Hide in range report"))


from xulpymoney.ui.myqtablewidget import myQTableWidget
from xulpymoney.ui.wdgYear import wdgYear
from xulpymoney.ui.wdgYearMonth import wdgYearMonth
import xulpymoney.images.xulpymoney_rc



import logging
from decimal import Decimal
from xulpymoney.libxulpymoney import AccountOperation, Comment, Money
from xulpymoney.libxulpymoneyfunctions import qdatetime, qleft
from xulpymoney.libmanagers import ObjectManager_With_IdDatetime_Selectable
from xulpymoney.libxulpymoneytypes import eMoneyCurrency, eConcept, eComment
from PyQt5.QtWidgets import QTableWidgetItem
from PyQt5.QtCore import QObject
## Class to manage CDF daily contracts.

class HlContract(QObject):
    def __init__(self, mem, investment):
        QObject.__init__(self)
        self.mem=mem
        self.init__create(None, None, Decimal('0'), Decimal('0'), Decimal('0'), Decimal('0'), None, None, None, None, 1)
        self.investment=investment

    def __repr__(self):
        return (self.tr("HLContract {} for {} at {}").format(self.id, self.investment.name,  self.datetime))


    ## Creates a copy of the current object. It's not a pointer to the object
    def copy(self):
        return HlContract(self.mem, self.investment).init__create(self.id, self.datetime, self.guarantee, self.adjustment, self.commission, self.interest, self.guarantee_ao, self.adjustment_ao, self.commission_ao, self.interest_ao, self.currency_conversion)

    def init__db_row(self,  row):
        return self.init__create(row['id'], row['datetime'], row['guarantee'], row['adjustment'], row['commission'], row['interest'], row['guarantee_ao'], row['adjustment_ao'], row['interest_ao'], row['commission_ao'], row['currency_conversion'])
        
    def init__create(self, id, dt, guarantee, adjustment, commission, interest, guarantee_ao, adjustment_ao, interest_ao, commission_ao, currency_conversion):
        self.id=id
        self.datetime=dt
        self.guarantee=guarantee
        self.adjustment=adjustment
        self.commission=commission
        self.interest=interest
        self.guarantee_ao=guarantee_ao
        self.adjustment_ao=adjustment_ao
        self.interest_ao=interest_ao
        self.commission_ao=commission_ao
        self.currency_conversion=currency_conversion
        return self
        
    def init__from_accountoperation(self, accountoperation):
        """AccountOperation is a object, and must have id_conceptos share of sale or purchase. 
        IO returned is an object already created in investments_all()"""
        cur=self.mem.con.cursor()
        cur.execute("select id_inversiones,id_operinversiones from opercuentasdeoperinversiones where id_opercuentas=%s", (accountoperation.id, ))
        if cur.rowcount==0:
            cur.close()
            return None
        row=cur.fetchone()
        cur.close()
        investment=self.mem.data.investments.find_by_id(row['id_inversiones'])
        return investment.op.find(row['id_operinversiones'])
        
    ## Returns a Money object with the self.guarantee amount
    def mGuarantee(self, type):
        money=Money(self.mem, self.guarantee, self.investment.product.currency)
        if type==eMoneyCurrency.Product:
            return money
        elif type==eMoneyCurrency.Account:
            return money.convert_from_factor(self.investment.account.currency, self.currency_conversion)
        elif type==eMoneyCurrency.User:
            return money.convert_from_factor(self.investment.account.currency, self.currency_conversion).local(self.datetime)
    
    ## Returns a Money object 
    def mAdjustment(self, type):
        money=Money(self.mem, self.adjustment, self.investment.product.currency)
        if type==eMoneyCurrency.Product:
            return money
        elif type==eMoneyCurrency.Account:
            return money.convert_from_factor(self.investment.account.currency, self.currency_conversion)
        elif type==eMoneyCurrency.User:
            return money.convert_from_factor(self.investment.account.currency, self.currency_conversion).local(self.datetime)

    ## Returns a Money object 
    def mCommission(self, type):
        money=Money(self.mem, self.commission, self.investment.product.currency)
        if type==eMoneyCurrency.Product:
            return money
        elif type==eMoneyCurrency.Account:
            return money.convert_from_factor(self.investment.account.currency, self.currency_conversion)
        elif type==eMoneyCurrency.User:
            return money.convert_from_factor(self.investment.account.currency, self.currency_conversion).local(self.datetime)
    
    ## Returns a Money object 
    def mInterest(self, type):
        money=Money(self.mem, self.interest, self.investment.product.currency)
        if type==eMoneyCurrency.Product:
            return money
        elif type==eMoneyCurrency.Account:
            return money.convert_from_factor(self.investment.account.currency, self.currency_conversion)
        elif type==eMoneyCurrency.User:
            return money.convert_from_factor(self.investment.account.currency, self.currency_conversion).local(self.datetime)

    ## Removes from database contract, and all operaccounts asociated
    def delete_from_db(self):
        cur=self.mem.con.cursor()
        cur.execute("delete from high_low_contract where id=%s", (self.id, ))
        cur.close()
        if self.guarantee_ao!=None:
            AO=AccountOperation(self.mem, self.guarantee_ao)
            AO.borrar()
        if self.adjustment_ao!=None:
            AO=AccountOperation(self.mem, self.adjustment_ao)
            AO.borrar()
        if self.interest_ao!=None:
            AO=AccountOperation(self.mem, self.interest_ao)
            AO.borrar()
        if self.commission_ao!=None:
            AO=AccountOperation(self.mem, self.commission_ao)
            AO.borrar()
        

    def find_by_mem(self, investment, id):
        """
            Searchs in mem (needed investments_all())
            invesment is an Investment object
            id is the invesmentoperation to find
        """
        for i in self.mem.data.investments:
            if investment.id==i.id:
                found=i.op.find(id)
                if found!=None:
                    return found
        logging.warning ("Investment operation {} hasn't been found in mem".format(id))
        return None

    ## Save this HlContract. If self.id==None inserts else updates.
    ## Database stores product amounts y product currency amount
    def save(self):
        # Use current _ao before changing them to delete at the end of the process
        old_guarantee_ao=self.guarantee_ao
        old_adjustment_ao=self.adjustment_ao
        old_interest_ao=self.interest_ao
        old_commission_ao=self.commission_ao

        # Save
        cur=self.mem.con.cursor()
        if self.id==None:#insertar
            cur.execute("insert into high_low_contract(datetime, investments_id, guarantee, adjustment, commission, interest, currency_conversion) values (%s, %s, %s, %s, %s, %s, %s) returning id", 
                (self.datetime, self.investment.id,  self.guarantee, self.adjustment, self.commission, self.interest, self.currency_conversion))
            self.id=cur.fetchone()[0]
            self.investment.hlcontractmanager.append(self)
        else:
            cur.execute("update high_low_contract set datetime=%s, guarantee=%s, adjustment=%s, commission=%s, interest=%s, currency_conversion=%s where id=%s", 
                (self.datetime,  self.guarantee, self.adjustment, self.commission, self.interest, self.currency_conversion, self.id))

        # Creates new AO
        comment=Comment(self.mem).encode(eComment.HlContract, self)
        if self.guarantee!=0:
            concepto=self.mem.conceptos.find_by_id(eConcept.HlGuaranteeReturned) if self.guarantee>0 else self.mem.conceptos.find_by_id(eConcept.HlGuaranteePaid)
            guarantee_AO=AccountOperation(self.mem, self.datetime, concepto, concepto.tipooperacion, self.mGuarantee(eMoneyCurrency.Account).amount, comment, self.investment.account, None)
            guarantee_AO.save()
            self.guarantee_ao=guarantee_AO.id
        else:
            self.guarantee_ao=None#To reset guarantee_ao in database and avoid fk errors
        if self.adjustment!=0:
            concepto=self.mem.conceptos.find_by_id(eConcept.HlAdjustmentIincome) if self.adjustment>0 else self.mem.conceptos.find_by_id(eConcept.HlAdjustmentExpense)
            adjustment_AO=AccountOperation(self.mem, self.datetime, concepto, concepto.tipooperacion, self.mAdjustment(eMoneyCurrency.Account).amount, comment, self.investment.account, None)
            adjustment_AO.save()
            self.adjustment_ao=adjustment_AO.id
        else:
            self.adjustment_ao=None#To reset guarantee_ao in database and avoid fk errors
        if self.interest!=0:
            concepto=self.mem.conceptos.find_by_id(eConcept.HlInterestPaid) if self.adjustment>0 else self.mem.conceptos.find_by_id(eConcept.HlInterestReceived)
            interest_AO=AccountOperation(self.mem, self.datetime, concepto, concepto.tipooperacion, self.mInterest(eMoneyCurrency.Account).amount, comment, self.investment.account, None)
            interest_AO.save()
            self.interest_ao=interest_AO.id
        else:
            self.interest_ao=None#To reset guarantee_ao in database and avoid fk errors
        if self.commission!=0:
            concepto=self.mem.conceptos.find_by_id(eConcept.HlCommission)
            commission_AO=AccountOperation(self.mem, self.datetime, concepto, concepto.tipooperacion, -self.mCommission(eMoneyCurrency.Account).amount, comment, self.investment.account, None)
            commission_AO.save()
            self.commission_ao=commission_AO.id
        else:
            self.commission_ao=None#To reset guarantee_ao in database and avoid fk errors

        cur.execute("update high_low_contract set guarantee_ao=%s, adjustment_ao=%s, interest_ao=%s, commission_ao=%s where id=%s", 
                (self.guarantee_ao, self.adjustment_ao, self.interest_ao, self.commission_ao, self.id))
        cur.close()

        #Delete old AO. This made me change foreign key to NO ACTION.NO ACTION allows the check to be deferred until later in the transaction
        if old_guarantee_ao!=None:
            AO=AccountOperation(self.mem, old_guarantee_ao)
            AO.borrar()
        if old_adjustment_ao!=None:
            AO=AccountOperation(self.mem, old_adjustment_ao)
            AO.borrar()
        if old_interest_ao!=None:
            AO=AccountOperation(self.mem, old_interest_ao)
            AO.borrar()
        if old_commission_ao!=None:
            AO=AccountOperation(self.mem, old_commission_ao)
            AO.borrar()

        self.investment.hlcontractmanager.order_by_datetime()

class HlContractManagerHeterogeneus(ObjectManager_With_IdDatetime_Selectable):
    def __init__(self, mem):
        ObjectManager_With_IdDatetime_Selectable.__init__(self)        
        self.mem=mem

    ## Returns the sum of guarantees
    def mGuarantees(self, type):
        currency=self.investment.resultsCurrency(type)
        r=Money(self.mem, 0, currency)
        for o in self.arr:
            r=r+o.mGuarantee(type)
        return r        
    ## Returns the sum of adjustments
    def mAdjustments(self, type):
        currency=self.investment.resultsCurrency(type)
        r=Money(self.mem, 0, currency)
        for o in self.arr:
            r=r+o.mAdjustment(type)
        return r        
    ## Returns the sum of interest
    def mInterests(self, type):
        currency=self.investment.resultsCurrency(type)
        r=Money(self.mem, 0, currency)
        for o in self.arr:
            r=r+o.mInterest(type)
        return r        
    ## Returns the sum of commissions
    def mCommissions(self, type):
        currency=self.investment.resultsCurrency(type)
        r=Money(self.mem, 0, currency)
        for o in self.arr:
            r=r+o.mCommission(type)
        return r
        
    ## Removes from array and from database
    def delete_from_db(self, obj):
        obj.delete_from_db()
        ObjectManager_With_IdDatetime_Selectable.remove(self, obj)
        
class HlContractManagerHomogeneus(HlContractManagerHeterogeneus, QObject):
    def __init__(self, mem, investment):
        QObject.__init__(self)
        HlContractManagerHeterogeneus.__init__(self, mem)
        self.investment=investment

    ## Function that acts like a constructor
    ## @param sql, to get from high_low_contract table, for example: select * from high_low_contract order by datetime;
    def init__from_db(self, sql):
        self.clean()
        cur=self.mem.con.cursor()
        cur.execute(sql)
        for row in cur:
            hl=HlContract(self.mem, self.investment).init__db_row(row)
            self.append(hl)
        cur.close()
        return self

    ## @param table
    ## @param type eMoneyCurrency 
    def myqtablewidget(self, table, type):
        table.setColumnCount(5)
        table.setHorizontalHeaderItem(0, QTableWidgetItem(self.tr("Date and time" )))
        table.setHorizontalHeaderItem(1, QTableWidgetItem(self.tr("Adjustment" )))
        table.setHorizontalHeaderItem(2, QTableWidgetItem(self.tr("Guarantee" )))
        table.setHorizontalHeaderItem(3, QTableWidgetItem(self.tr("Interest" )))
        table.setHorizontalHeaderItem(4, QTableWidgetItem(self.tr("Commission" )))
        table.applySettings()
        table.clearContents()
        table.setRowCount(self.length()+1)
        for i, o in enumerate(self.arr):
            table.setItem(i, 0, qdatetime(o.datetime, self.mem.localzone))
            table.setItem(i, 1, o.mAdjustment(type).qtablewidgetitem())
            table.setItem(i, 2, o.mGuarantee(type).qtablewidgetitem())
            table.setItem(i, 3, o.mInterest(type).qtablewidgetitem())
            table.setItem(i, 4, o.mCommission(type).qtablewidgetitem())
        table.setItem(self.length(), 0, qleft(self.tr("TOTAL")))
        table.setItem(self.length(), 1, self.mAdjustments(type).qtablewidgetitem())
        table.setItem(self.length(), 2, self.mGuarantees(type).qtablewidgetitem())
        table.setItem(self.length(), 3, self.mInterests(type).qtablewidgetitem())
        table.setItem(self.length(), 4, self.mCommissions(type).qtablewidgetitem())



import datetime
from PyQt5.QtCore import pyqtSlot
from PyQt5.QtWidgets import QMenu, QWidget, QHBoxLayout,  QTableWidgetItem
from xulpymoney.ui.myqtablewidget import myQTableWidget
from xulpymoney.libxulpymoney import AccountOperationManager
from xulpymoney.libxulpymoneyfunctions import qcenter, qmessagebox
from xulpymoney.ui.Ui_wdgConceptsHistorical import Ui_wdgConceptsHistorical

class wdgConceptsHistorical(QWidget, Ui_wdgConceptsHistorical):
    def __init__(self, mem, concepto,  parent=None):
        QWidget.__init__(self, parent)
        self.setupUi(self)
        self.mem=mem
        self.concepto=concepto

        self.month=None#Used to show popup with month or year report if is 0->Year, else->Month
        self.year=None
        self.firstyear=None
        self.table.settings(self.mem, "wdgConceptsHistorical")
        self.reload()

    def reload(self):
        cur=self.mem.con.cursor()
        #Junta opercuentas y opertarjetas y sobre esa subquery uni hace un group by
        sql="""
        select date_part('year',datetime) as year,  date_part('month',datetime) as month, sum(importe) as suma 
        from ( 
                    SELECT opercuentas.datetime, opercuentas.id_conceptos,  opercuentas.importe  FROM opercuentas where id_conceptos={0} 
                        UNION ALL 
                    SELECT opertarjetas.datetime, opertarjetas.id_conceptos, opertarjetas.importe FROM opertarjetas where id_conceptos={0}
                ) as uni 
        group by date_part('year',datetime), date_part('month',datetime) order by 1,2 ;
        """.format(self.concepto.id)
        cur.execute(sql)
        if cur.rowcount!=0:
            arr=cur.fetchall()            
        cur.close()
        self.table.applySettings()
        #Coloca filas y años
        self.firstyear=int(arr[0][0])
        rows=int(datetime.date.today().year-self.firstyear+1)
        self.table.setRowCount(rows+1)
        suma=[]#sumaa en array suma, en el que el año first es 0...
        for i in range(rows):
            suma.append(0)#Inicializa a 0 los sumadores
            self.table.setItem(i, 0, QTableWidgetItem("{0}".format(self.firstyear+i)))
        #Coloca en tabla 
        for a in arr:
            self.table.setItem(a[0]-self.firstyear, a[1],self.mem.localcurrency.qtablewidgetitem(a[2])  )
            suma[int(a[0])-self.firstyear]=suma[int(a[0])-self.firstyear]+a[2]
        #Coloca en tabla los sumaatorios
        for i,  s in enumerate(suma):
            self.table.setItem(i, 13,self.mem.localcurrency.qtablewidgetitem(s) )

        #Add years total
        self.table.setItem(rows, 0, qcenter(self.tr("Total")))
        self.table.setItem(rows, 13, self.mem.localcurrency.qtablewidgetitem(sum(suma)))

    @pyqtSlot() 
    def on_actionShowMonth_triggered(self):
        newtab = QWidget()
        horizontalLayout = QHBoxLayout(newtab)
        table = myQTableWidget(newtab)
        table.settings(self.mem, "wdgConceptsHistorical",  "tblShowMonth")
        set=AccountOperationManager(self.mem)
        set.load_from_db_with_creditcard("""
             select datetime, id_conceptos, id_tiposoperaciones, importe, comentario, id_cuentas , -1 as id_tarjetas 
             from opercuentas 
             where
                 id_conceptos={0} and 
                 date_part('year',datetime)={1} and 
                 date_part('month',datetime)={2} 
             union all 
             select datetime, id_conceptos, id_tiposoperaciones, importe, comentario, id_cuentas ,tarjetas.id_tarjetas as id_tarjetas 
             from opertarjetas, tarjetas 
             where 
                 opertarjetas.id_tarjetas=tarjetas.id_tarjetas and 
                 id_conceptos={0} and 
                 date_part('year',datetime)={1} and 
                 date_part('month',datetime)={2}""".format (self.concepto.id, self.year, self.month))
        set.myqtablewidget(table, True)
        horizontalLayout.addWidget(table)
        self.tab.addTab(newtab, self.tr("Report of {0} of {1}".format(self.table.horizontalHeaderItem(self.month).text(), self.year)))
        self.tab.setCurrentWidget(newtab)

    @pyqtSlot() 
    def on_actionShowYear_triggered(self):
        newtab = QWidget()
        horizontalLayout = QHBoxLayout(newtab)
        table = myQTableWidget(newtab)
        table.settings(self.mem, "wdgConceptsHistorical",  "tblShowYear")
        set=AccountOperationManager(self.mem)
        set.load_from_db_with_creditcard("select datetime, id_conceptos, id_tiposoperaciones, importe, comentario, id_cuentas , -1 as id_tarjetas from opercuentas where id_conceptos={0} and date_part('year',datetime)={1} union all select datetime, id_conceptos, id_tiposoperaciones, importe, comentario, id_cuentas ,tarjetas.id_tarjetas as id_tarjetas from opertarjetas,tarjetas where opertarjetas.id_tarjetas=tarjetas.id_tarjetas and id_conceptos={0} and date_part('year',datetime)={1}".format (self.concepto.id, self.year))
        set.sort()
        set.myqtablewidget(table, True)
        horizontalLayout.addWidget(table)
        self.tab.addTab(newtab, self.tr("Report of {0}".format(self.year)))
        self.tab.setCurrentWidget(newtab)

    def on_table_customContextMenuRequested(self,  pos):
        self.actionShowYear.setEnabled(False)
        self.actionShowMonth.setEnabled(False)
        if self.month!=None:
            if self.month==0 and self.year<=datetime.date.today().year:#Avoid total:
                self.actionShowYear.setEnabled(True)
            elif self.month>0:
                self.actionShowMonth.setEnabled(True)

        menu=QMenu()
        menu.addAction(self.actionShowYear)
        menu.addSeparator()
        menu.addAction(self.actionShowMonth)      
        menu.exec_(self.table.mapToGlobal(pos))

    def on_table_itemSelectionChanged(self):
        self.month=None
        self.year=None
        for i in self.table.selectedItems():#itera por cada item no row.
            if i.column()==0 or i.column()==13:
                self.month=0
            else:
                self.month=i.column()
            self.year=self.firstyear+i.row()
        print ("Selected year: {0}. Selected month: {1}.".format(self.year, self.month))

    def on_tab_tabCloseRequested(self, index):
        """Only removes dinamic tabs"""
        if index==0:
            qmessagebox(self.tr("You can't close this tab"))
        else:
            self.tab.removeTab(index)



from PyQt5.QtWidgets import QDialog
from xulpymoney.ui.Ui_frmCreditCardsAdd import Ui_frmCreditCardsAdd
from xulpymoney.libxulpymoney import CreditCard
from xulpymoney.libxulpymoneyfunctions import b2c, c2b, qmessagebox

class frmCreditCardsAdd(QDialog, Ui_frmCreditCardsAdd):
    def __init__(self, mem,  account,  creditcard,  parent=None):
        """
            Account es registro
            Si creditcard=None # Insertar
            Si creditcard=reg(CreditCard) #Modificar
        """
        QDialog.__init__(self, parent)
        self.setupUi(self)
        self.mem=mem
        self.account=account

        if creditcard==None:
            self.tipo=1#Insertar
            self.creditcard=CreditCard(self.mem)
            self.lblTitle.setText(self.tr("New credit card of {0}").format(self.account.name))
            self.creditcard.active=True
        else:
            self.tipo=2#Modificar
            self.creditcard=creditcard
            self.lblTitle.setText(self.tr("Updating {} credit card").format(self.creditcard.name))
            self.txtCreditCard.setText(self.creditcard.name)
            self.chkDelayed.setChecked(b2c(self.creditcard.pagodiferido))
            self.txtMaximum.setText(str(self.creditcard.saldomaximo))
            self.txtNumber.setText(self.creditcard.numero)
            
    def on_cmd_pressed(self):
        if self.txtMaximum.isValid():
            self.creditcard.name=self.txtCreditCard.text()
            self.creditcard.account=self.account
            self.creditcard.pagodiferido=c2b(self.chkDelayed.checkState())
            self.creditcard.saldomaximo=self.txtMaximum.decimal()
            self.creditcard.numero=self.txtNumber.text()
            self.creditcard.save()
            self.mem.con.commit()        
            
            if self.tipo==1:#insertar
                self.mem.data.creditcards.append(self.creditcard)
        
            self.done(0)
        else:
            qmessagebox(self.tr("You have written and invalid number"))
    



from PyQt5.QtWidgets import QWidget,  QTableWidgetItem,  QMenu
from PyQt5.QtCore import pyqtSlot
from xulpymoney.ui.Ui_wdgInvestmentsRanking import Ui_wdgInvestmentsRanking
from xulpymoney.ui.frmInvestmentReport import frmInvestmentReport
from xulpymoney.ui.frmProductReport import frmProductReport
from xulpymoney.libxulpymoney import Money

class wdgInvestmentsRanking(QWidget, Ui_wdgInvestmentsRanking):
    def __init__(self, mem,  parent=None):
        QWidget.__init__(self, parent)
        self.setupUi(self)
        self.mem=mem
        self.selOperations=None
        self.selCurrentOperations=None

        self.tab.setCurrentIndex(0)
        self.tblOperations.settings(self.mem,"wdgInvestmentsRanking" , "self.tblOperations")
        self.tblOperations.setColumnCount(5)
        self.tblOperations.setHorizontalHeaderItem(0, QTableWidgetItem(self.tr("Investment")))
        self.tblOperations.setHorizontalHeaderItem(1, QTableWidgetItem(self.tr("Current gains")))
        self.tblOperations.setHorizontalHeaderItem(2, QTableWidgetItem(self.tr("Historical gains")))
        self.tblOperations.setHorizontalHeaderItem(3, QTableWidgetItem(self.tr("Dividends")))
        self.tblOperations.setHorizontalHeaderItem(4, QTableWidgetItem(self.tr("Total")))

        set=self.mem.data.investments.setInvestments_merging_investments_with_same_product_merging_operations()
        self.tblOperations.applySettings()
        self.tblOperations.clearContents()
        self.tblOperations.setRowCount(set.length()+1)
        self.listOperations=[]
        sumcurrent=Money(self.mem, 0, self.mem.localcurrency)
        sumhistorical=Money(self.mem, 0, self.mem.localcurrency)
        sumdividends=Money(self.mem, 0, self.mem.localcurrency)
        for i, inv in enumerate(set.arr):
            gains_current=inv.op_actual.pendiente(inv.product.result.basic.last, type=3)
            gains_historical=inv.op_historica.consolidado_bruto(type=3)
            dividends=self.mem.data.investments.setDividends_merging_operation_dividends(inv.product)
            dividends_gross=dividends.gross(type=3)
            total=gains_current+gains_historical+dividends_gross
            sumcurrent=sumcurrent+gains_current
            sumhistorical=sumhistorical+gains_historical
            sumdividends=sumdividends+dividends_gross
            self.listOperations.append((inv, gains_current, gains_historical, dividends_gross, total))

        self.listOperations=sorted(self.listOperations, key=lambda c: c[4],  reverse=True)     

        for i, l in enumerate(self.listOperations):
            self.tblOperations.setItem(i, 0, QTableWidgetItem(l[0].product.name))
            self.tblOperations.setItem(i, 1, l[1].qtablewidgetitem())
            self.tblOperations.setItem(i, 2,  l[2].qtablewidgetitem())
            self.tblOperations.setItem(i, 3, l[3].qtablewidgetitem())
            self.tblOperations.setItem(i, 4, l[4].qtablewidgetitem())
        self.tblOperations.setItem(len(self.listOperations)+1, 0, QTableWidgetItem(self.tr("Total")))
        self.tblOperations.setItem(len(self.listOperations)+1, 1, sumcurrent.qtablewidgetitem())
        self.tblOperations.setItem(len(self.listOperations)+1, 2, sumhistorical.qtablewidgetitem())
        self.tblOperations.setItem(len(self.listOperations)+1, 3, sumdividends.qtablewidgetitem())
        self.tblOperations.setItem(len(self.listOperations)+1, 4, (sumcurrent+sumhistorical+sumdividends).qtablewidgetitem())

#####################################################################################################################33

        self.tblCurrentOperations.settings(self.mem,"wdgInvestmentsRanking" , "self.tblCurrentOperations")
        self.tblCurrentOperations.setColumnCount(5)
        self.tblCurrentOperations.setHorizontalHeaderItem(0, QTableWidgetItem(self.tr("Investment")))
        self.tblCurrentOperations.setHorizontalHeaderItem(1, QTableWidgetItem(self.tr("Current gains")))
        self.tblCurrentOperations.setHorizontalHeaderItem(2, QTableWidgetItem(self.tr("Historical gains")))
        self.tblCurrentOperations.setHorizontalHeaderItem(3, QTableWidgetItem(self.tr("Dividends")))
        self.tblCurrentOperations.setHorizontalHeaderItem(4, QTableWidgetItem(self.tr("Total")))

        self.tblCurrentOperations.applySettings()
        self.tblCurrentOperations.clearContents()
        self.tblCurrentOperations.setRowCount(set.length()+1)
        self.listCurrentOperations=[]
        sumcurrent=Money(self.mem, 0, self.mem.localcurrency)
        sumhistorical=Money(self.mem, 0, self.mem.localcurrency)
        sumdividends=Money(self.mem, 0, self.mem.localcurrency)
        for product in self.mem.data.investments.ProductManager_with_investments_distinct_products().arr:
            current=Money(self.mem, 0, self.mem.localcurrency)
            historical=Money(self.mem, 0, self.mem.localcurrency)
            dividends=Money(self.mem, 0, self.mem.localcurrency)

            for inv in self.mem.data.investments.arr:
                if inv.product.id==product.id:
                    current=current+inv.op_actual.pendiente(inv.product.result.basic.last, 3)
                    historical=historical+inv.op_historica.consolidado_bruto(type=3)
                    dividends=dividends+inv.setDividends_from_operations().gross(type=3)
            sumcurrent=sumcurrent+current
            sumhistorical=sumhistorical+historical
            sumdividends=sumdividends+dividends
            self.listCurrentOperations.append((product, current, historical, dividends, current+historical+dividends))

        self.listCurrentOperations=sorted(self.listCurrentOperations, key=lambda c: c[4],  reverse=True)     

        for i, l in enumerate(self.listCurrentOperations):
            self.tblCurrentOperations.setItem(i, 0, QTableWidgetItem(l[0].name))
            self.tblCurrentOperations.setItem(i, 1, l[1].qtablewidgetitem())
            self.tblCurrentOperations.setItem(i, 2,  l[2].qtablewidgetitem())
            self.tblCurrentOperations.setItem(i, 3, l[3].qtablewidgetitem())
            self.tblCurrentOperations.setItem(i, 4, l[4].qtablewidgetitem())
        self.tblCurrentOperations.setItem(len(self.listCurrentOperations)+1, 0, QTableWidgetItem(self.tr("Total")))
        self.tblCurrentOperations.setItem(len(self.listCurrentOperations)+1, 1, sumcurrent.qtablewidgetitem())
        self.tblCurrentOperations.setItem(len(self.listCurrentOperations)+1, 2, sumhistorical.qtablewidgetitem())
        self.tblCurrentOperations.setItem(len(self.listCurrentOperations)+1, 3, sumdividends.qtablewidgetitem())
        self.tblCurrentOperations.setItem(len(self.listCurrentOperations)+1, 4, (sumcurrent+sumhistorical+sumdividends).qtablewidgetitem())

    @pyqtSlot() 
    def on_actionSameProduct_triggered(self):
        inv=self.mem.data.investments.investment_merging_current_operations_with_same_product(self.selCurrentOperations)
        w=frmInvestmentReport(self.mem, inv, self)
        w.exec_()

    @pyqtSlot() 
    def on_actionSameProductFIFO_triggered(self):
        w=frmInvestmentReport(self.mem, self.selOperations, self)
        w.exec_()

    @pyqtSlot() 
    def on_actionProduct_triggered(self):
        if self.tab.currentIndex()==0:
            product=self.selCurrentOperations
            inv=self.mem.data.investments.investment_merging_current_operations_with_same_product(self.selCurrentOperations)
        else:
            product=self.selOperations.product
            inv=self.selOperations

        w=frmProductReport(self.mem, product, inv, self)
        w.exec_()

    def on_tblOperations_customContextMenuRequested(self,  pos):
        if self.selOperations==None:
            self.actionProduct.setEnabled(False)
            self.actionSameProductFIFO.setEnabled(False)
        else:
            self.actionProduct.setEnabled(True)
            self.actionSameProductFIFO.setEnabled(True)

        menu=QMenu()
        menu.addAction(self.actionProduct)
        menu.addSeparator()
        menu.addAction(self.actionSameProductFIFO)       
        menu.exec_(self.tblOperations.mapToGlobal(pos))

    def on_tblOperations_itemSelectionChanged(self):
        self.selOperations=None
        try:
            for i in self.tblOperations.selectedItems():#itera por cada item no row.
                self.selOperations=self.listOperations[i.row()][0]
        except:
            pass
            
            
        
    def on_tblCurrentOperations_customContextMenuRequested(self,  pos):
        if self.selCurrentOperations==None:
            self.actionProduct.setEnabled(False)
            self.actionSameProduct.setEnabled(False)
        else:
            self.actionProduct.setEnabled(True)
            self.actionSameProduct.setEnabled(True)

        menu=QMenu()
        menu.addAction(self.actionProduct)
        menu.addSeparator()
        menu.addAction(self.actionSameProduct)
        menu.exec_(self.tblCurrentOperations.mapToGlobal(pos))

    def on_tblCurrentOperations_itemSelectionChanged(self):
        self.selCurrentOperations=None
        try:
            for i in self.tblCurrentOperations.selectedItems():#itera por cada item no row.
                self.selCurrentOperations=self.listCurrentOperations[i.row()][0]
        except:
            pass



import colorama
import officegenerator
import platform
import stdnum
import PyQt5.QtCore
import PyQt5.QtChart
from PyQt5.QtWidgets import QDialog
from xulpymoney.libxulpymoneyfunctions import qcenter, qempty, qright
from xulpymoney.libxulpymoneytypes import eProductType
from xulpymoney.ui.Ui_frmAbout import Ui_frmAbout

class frmAbout(QDialog, Ui_frmAbout):
    def __init__(self, mem,  parent = None, name = None, modal = False):
        """
        Constructor
        
        @param parent The parent widget of this dialog. (QWidget)
        @param name The name of this dialog. (QString)
        @param modal Flag indicating a modal dialog. (boolean)
        """
        self.mem=mem
        QDialog.__init__(self, parent)
        if name:
            self.setObjectName(name)
        self.setModal(True)
        self.setupUi(self)
        
        self.tblSoftware.settings(self.mem, "frmAbout")
        self.tblStatistics.settings(self.mem, "frmAbout")
        self.load_tblStatistics() 
        self.load_tblSoftware()
        self.tblStatistics.applySettings()    
    
    def load_tblStatistics(self):
        def pais(cur, columna, bolsa):
            """Si pais es Null es para todos"""
            total=0
            cur.execute("select count(*) from products where type=1 and obsolete=false and stockmarkets_id=%s", (bolsa.id,))
            tmp=cur.fetchone()[0]
            total=total+tmp
            self.tblStatistics.setItem(0, columna , qcenter(tmp))
            cur.execute("select count(*) from products where type=2 and obsolete=false and stockmarkets_id=%s", (bolsa.id,))
            tmp=cur.fetchone()[0]
            total=total+tmp
            self.tblStatistics.setItem(1, columna , qcenter(tmp))
            cur.execute("select count(*) from products where type=3 and obsolete=false and stockmarkets_id=%s", (bolsa.id,))
            tmp=cur.fetchone()[0]
            total=total+tmp
            self.tblStatistics.setItem(2, columna , qcenter(tmp))
            cur.execute("select count(*) from products where type=4 and obsolete=false and stockmarkets_id=%s", (bolsa.id,))
            tmp=cur.fetchone()[0]
            total=total+tmp
            self.tblStatistics.setItem(3, columna , qcenter(tmp))
            cur.execute("select count(*) from products where type=5 and obsolete=false and stockmarkets_id=%s", (bolsa.id,))
            tmp=cur.fetchone()[0]
            total=total+tmp
            self.tblStatistics.setItem(4, columna , qcenter(tmp))
            cur.execute("select count(*) from products where type=7 and obsolete=false and stockmarkets_id=%s", (bolsa.id,))
            tmp=cur.fetchone()[0]
            total=total+tmp
            self.tblStatistics.setItem(5, columna , qcenter(tmp))
            cur.execute("select count(*) from products where type=9 and obsolete=false and stockmarkets_id=%s", (bolsa.id,))
            tmp=cur.fetchone()[0]
            total=total+tmp
            self.tblStatistics.setItem(6, columna , qcenter(tmp))
            cur.execute("select count(*) from products where type=%s and obsolete=false and stockmarkets_id=%s", (eProductType.CFD.value, bolsa.id,))
            tmp=cur.fetchone()[0]
            total=total+tmp
            self.tblStatistics.setItem(7, columna , qcenter(tmp))
            cur.execute("select count(*) from products where type=%s and obsolete=false and stockmarkets_id=%s", (eProductType.Future.value, bolsa.id,))
            tmp=cur.fetchone()[0]
            total=total+tmp
            self.tblStatistics.setItem(8, columna , qcenter(tmp))
            self.tblStatistics.setItem(9, columna , qempty())
            cur.execute("select count(*) from products where obsolete=true and stockmarkets_id=%s", (bolsa.id,))
            tmp=cur.fetchone()[0]
            self.tblStatistics.setItem(10, columna , qcenter(tmp))
            self.tblStatistics.setItem(11, columna , qempty())
            self.tblStatistics.setItem(12, columna , qcenter(total))
            self.tblStatistics.horizontalHeaderItem (columna).setIcon(bolsa.country.qicon())
            self.tblStatistics.horizontalHeaderItem (columna).setToolTip((bolsa.country.name))

        def todos(cur):
            """Si pais es Null es para todos"""
            total=0
            cur.execute("select count(*) from products where type=1 and obsolete=false")
            tmp=cur.fetchone()[0]
            total=total+tmp
            self.tblStatistics.setItem(0, 0 , qcenter(tmp))
            cur.execute("select count(*) from products where type=2  and obsolete=false")
            tmp=cur.fetchone()[0]
            total=total+tmp
            self.tblStatistics.setItem(1, 0 , qcenter(tmp))
            cur.execute("select count(*) from products where type=3  and obsolete=false")
            tmp=cur.fetchone()[0]
            total=total+tmp
            self.tblStatistics.setItem(2, 0 , qcenter(tmp))
            cur.execute("select count(*) from products where type=4  and obsolete=false")
            tmp=cur.fetchone()[0]
            total=total+tmp
            self.tblStatistics.setItem(3, 0 , qcenter(tmp))
            cur.execute("select count(*) from products where type=5  and obsolete=false")
            tmp=cur.fetchone()[0]
            total=total+tmp
            self.tblStatistics.setItem(4, 0 , qcenter(tmp))
            cur.execute("select count(*) from products where type=7  and obsolete=false")
            tmp=cur.fetchone()[0]
            total=total+tmp
            self.tblStatistics.setItem(5, 0 , qcenter(tmp))
            cur.execute("select count(*) from products where type=9  and obsolete=false")
            tmp=cur.fetchone()[0]
            total=total+tmp
            self.tblStatistics.setItem(6, 0 , qcenter(tmp))
            cur.execute("select count(*) from products where type=%s  and obsolete=false", (eProductType.CFD.value, ))
            tmp=cur.fetchone()[0]
            total=total+tmp
            self.tblStatistics.setItem(7, 0 , qcenter(tmp))
            cur.execute("select count(*) from products where type=%s  and obsolete=false", (eProductType.Future.value, ))
            tmp=cur.fetchone()[0]
            total=total+tmp
            self.tblStatistics.setItem(8, 0 , qcenter(tmp))
            self.tblStatistics.setItem(9, 0 , qempty())
            cur.execute("select count(*) from products where obsolete=true ")
            tmp=cur.fetchone()[0]
            self.tblStatistics.setItem(10, 0 , qcenter(tmp))
            self.tblStatistics.setItem(11, 0 , qempty())
            self.tblStatistics.setItem(12, 0 , qcenter(total))

    
        cur = self.mem.con.cursor()
        todos(cur)
        pais(cur, 1, self.mem.stockmarkets.find_by_id(1))
        pais(cur, 2, self.mem.stockmarkets.find_by_id(2))
        pais(cur, 3, self.mem.stockmarkets.find_by_id(3))
        pais(cur, 4, self.mem.stockmarkets.find_by_id(4))
        pais(cur, 5, self.mem.stockmarkets.find_by_id(5))
        pais(cur, 6,self.mem.stockmarkets.find_by_id(6))
        pais(cur, 7, self.mem.stockmarkets.find_by_id(7))
        pais(cur, 8, self.mem.stockmarkets.find_by_id(8))
        pais(cur, 9, self.mem.stockmarkets.find_by_id(9))
        pais(cur, 10, self.mem.stockmarkets.find_by_id(10))
        pais(cur, 11, self.mem.stockmarkets.find_by_id(11))
        pais(cur, 12, self.mem.stockmarkets.find_by_id(12))
        pais(cur, 13, self.mem.stockmarkets.find_by_id(13))
        pais(cur, 14, self.mem.stockmarkets.find_by_id(14))
        pais(cur, 15, self.mem.stockmarkets.find_by_id(15))
        cur.close()

    ##Function that fills tblSoftware with data 
    def load_tblSoftware(self):
        self.tblSoftware.setItem(0, 0 , qright(colorama.__version__))
        self.tblSoftware.setItem(1, 0 , qright(officegenerator.__version__))
        self.tblSoftware.setItem(2, 0 , qright(PyQt5.QtCore.PYQT_VERSION_STR))
        self.tblSoftware.setItem(3, 0 , qright(PyQt5.QtChart.PYQT_CHART_VERSION_STR))
        self.tblSoftware.setItem(4, 0 , qright(platform.python_version()))
        self.tblSoftware.setItem(5, 0, qright(stdnum.__version__))



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'xulpymoney/ui/frmTransfer.ui'
#
# Created by: PyQt5 UI code generator 5.12.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_frmTransfer(object):
    def setupUi(self, frmTransfer):
        frmTransfer.setObjectName("frmTransfer")
        frmTransfer.setWindowModality(QtCore.Qt.WindowModal)
        frmTransfer.resize(612, 246)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/xulpymoney/transfer.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        frmTransfer.setWindowIcon(icon)
        frmTransfer.setModal(True)
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout(frmTransfer)
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.lblTitulo = QtWidgets.QLabel(frmTransfer)
        font = QtGui.QFont()
        font.setPointSize(14)
        font.setBold(True)
        font.setWeight(75)
        self.lblTitulo.setFont(font)
        self.lblTitulo.setStyleSheet("color: rgb(0, 128, 0);")
        self.lblTitulo.setAlignment(QtCore.Qt.AlignCenter)
        self.lblTitulo.setObjectName("lblTitulo")
        self.verticalLayout.addWidget(self.lblTitulo)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.wdgDT = wdgDatetime(frmTransfer)
        self.wdgDT.setObjectName("wdgDT")
        self.horizontalLayout_2.addWidget(self.wdgDT)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.label_2 = QtWidgets.QLabel(frmTransfer)
        self.label_2.setObjectName("label_2")
        self.horizontalLayout_3.addWidget(self.label_2)
        self.cmbOrigen = QtWidgets.QComboBox(frmTransfer)
        self.cmbOrigen.setObjectName("cmbOrigen")
        self.horizontalLayout_3.addWidget(self.cmbOrigen)
        self.verticalLayout.addLayout(self.horizontalLayout_3)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.label_5 = QtWidgets.QLabel(frmTransfer)
        self.label_5.setObjectName("label_5")
        self.horizontalLayout_5.addWidget(self.label_5)
        self.cmbDestino = QtWidgets.QComboBox(frmTransfer)
        self.cmbDestino.setObjectName("cmbDestino")
        self.horizontalLayout_5.addWidget(self.cmbDestino)
        self.verticalLayout.addLayout(self.horizontalLayout_5)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label_3 = QtWidgets.QLabel(frmTransfer)
        self.label_3.setObjectName("label_3")
        self.horizontalLayout.addWidget(self.label_3)
        self.txtImporte = myQLineEdit(frmTransfer)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.txtImporte.sizePolicy().hasHeightForWidth())
        self.txtImporte.setSizePolicy(sizePolicy)
        self.txtImporte.setObjectName("txtImporte")
        self.horizontalLayout.addWidget(self.txtImporte)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.label_4 = QtWidgets.QLabel(frmTransfer)
        self.label_4.setObjectName("label_4")
        self.horizontalLayout_4.addWidget(self.label_4)
        self.txtComision = myQLineEdit(frmTransfer)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.txtComision.sizePolicy().hasHeightForWidth())
        self.txtComision.setSizePolicy(sizePolicy)
        self.txtComision.setObjectName("txtComision")
        self.horizontalLayout_4.addWidget(self.txtComision)
        self.verticalLayout.addLayout(self.horizontalLayout_4)
        self.cmd = QtWidgets.QPushButton(frmTransfer)
        self.cmd.setIcon(icon)
        self.cmd.setObjectName("cmd")
        self.verticalLayout.addWidget(self.cmd)
        self.horizontalLayout_6.addLayout(self.verticalLayout)

        self.retranslateUi(frmTransfer)
        QtCore.QMetaObject.connectSlotsByName(frmTransfer)

    def retranslateUi(self, frmTransfer):
        _translate = QtCore.QCoreApplication.translate
        frmTransfer.setWindowTitle(_translate("frmTransfer", "Account transfer"))
        self.lblTitulo.setText(_translate("frmTransfer", "Transfer"))
        self.label_2.setText(_translate("frmTransfer", "Select origin account"))
        self.label_5.setText(_translate("frmTransfer", "Select destiny account"))
        self.label_3.setText(_translate("frmTransfer", "Amount"))
        self.txtImporte.setToolTip(_translate("frmTransfer", "Amount must be positive"))
        self.txtImporte.setText(_translate("frmTransfer", "0"))
        self.label_4.setText(_translate("frmTransfer", "Comission"))
        self.txtComision.setToolTip(_translate("frmTransfer", "Amount must be positive"))
        self.txtComision.setText(_translate("frmTransfer", "0"))
        self.cmd.setText(_translate("frmTransfer", "Transfer"))


from xulpymoney.ui.myqlineedit import myQLineEdit
from xulpymoney.ui.wdgDatetime import wdgDatetime
import xulpymoney.images.xulpymoney_rc



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'xulpymoney/ui/frmSelector.ui'
#
# Created by: PyQt5 UI code generator 5.12.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_frmSelector(object):
    def setupUi(self, frmSelector):
        frmSelector.setObjectName("frmSelector")
        frmSelector.resize(687, 352)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/xulpymoney/books.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        frmSelector.setWindowIcon(icon)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(frmSelector)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.lbl = QtWidgets.QLabel(frmSelector)
        font = QtGui.QFont()
        font.setPointSize(12)
        self.lbl.setFont(font)
        self.lbl.setStyleSheet("color: rgb(0, 192, 0);")
        self.lbl.setAlignment(QtCore.Qt.AlignCenter)
        self.lbl.setTextInteractionFlags(QtCore.Qt.LinksAccessibleByMouse|QtCore.Qt.TextSelectableByMouse)
        self.lbl.setObjectName("lbl")
        self.verticalLayout_2.addWidget(self.lbl)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.tblSelected = QtWidgets.QTableWidget(frmSelector)
        self.tblSelected.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.tblSelected.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.tblSelected.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.tblSelected.setObjectName("tblSelected")
        self.tblSelected.setColumnCount(2)
        self.tblSelected.setRowCount(0)
        item = QtWidgets.QTableWidgetItem()
        self.tblSelected.setHorizontalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblSelected.setHorizontalHeaderItem(1, item)
        self.tblSelected.horizontalHeader().setStretchLastSection(True)
        self.tblSelected.verticalHeader().setVisible(False)
        self.horizontalLayout.addWidget(self.tblSelected)
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.cmdUp = QtWidgets.QToolButton(frmSelector)
        self.cmdUp.setObjectName("cmdUp")
        self.verticalLayout.addWidget(self.cmdUp)
        self.cmdRight = QtWidgets.QToolButton(frmSelector)
        self.cmdRight.setObjectName("cmdRight")
        self.verticalLayout.addWidget(self.cmdRight)
        self.cmdLeft = QtWidgets.QToolButton(frmSelector)
        self.cmdLeft.setObjectName("cmdLeft")
        self.verticalLayout.addWidget(self.cmdLeft)
        self.cmdDown = QtWidgets.QToolButton(frmSelector)
        self.cmdDown.setObjectName("cmdDown")
        self.verticalLayout.addWidget(self.cmdDown)
        self.horizontalLayout.addLayout(self.verticalLayout)
        self.tbl = QtWidgets.QTableWidget(frmSelector)
        self.tbl.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.tbl.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.tbl.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.tbl.setObjectName("tbl")
        self.tbl.setColumnCount(2)
        self.tbl.setRowCount(0)
        item = QtWidgets.QTableWidgetItem()
        self.tbl.setHorizontalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        self.tbl.setHorizontalHeaderItem(1, item)
        self.tbl.horizontalHeader().setStretchLastSection(True)
        self.tbl.verticalHeader().setVisible(False)
        self.horizontalLayout.addWidget(self.tbl)
        self.verticalLayout_2.addLayout(self.horizontalLayout)
        self.cmd = QtWidgets.QPushButton(frmSelector)
        self.cmd.setObjectName("cmd")
        self.verticalLayout_2.addWidget(self.cmd)
        self.horizontalLayout_2.addLayout(self.verticalLayout_2)

        self.retranslateUi(frmSelector)
        QtCore.QMetaObject.connectSlotsByName(frmSelector)

    def retranslateUi(self, frmSelector):
        _translate = QtCore.QCoreApplication.translate
        self.lbl.setText(_translate("frmSelector", "Investment"))
        item = self.tblSelected.horizontalHeaderItem(0)
        item.setText(_translate("frmSelector", "Id"))
        item = self.tblSelected.horizontalHeaderItem(1)
        item.setText(_translate("frmSelector", "Selected elements"))
        self.cmdUp.setText(_translate("frmSelector", "^"))
        self.cmdRight.setText(_translate("frmSelector", ">"))
        self.cmdLeft.setText(_translate("frmSelector", "<"))
        self.cmdDown.setText(_translate("frmSelector", "V"))
        item = self.tbl.horizontalHeaderItem(0)
        item.setText(_translate("frmSelector", "Id"))
        item = self.tbl.horizontalHeaderItem(1)
        item.setText(_translate("frmSelector", "Elements"))
        self.cmd.setText(_translate("frmSelector", "Accept selection"))


import xulpymoney.images.xulpymoney_rc



from PyQt5.QtCore import Qt,  pyqtSlot
from PyQt5.QtWidgets import QDialog
from xulpymoney.ui.Ui_frmQuotesIBM import Ui_frmQuotesIBM
from xulpymoney.libxulpymoney import Quote
from xulpymoney.libxulpymoneyfunctions import dtaware, dtaware_changes_tz, qmessagebox
from xulpymoney.libxulpymoneytypes import eProductType

class frmQuotesIBM(QDialog, Ui_frmQuotesIBM):
    def __init__(self, mem, product,  quote=None,   parent = None):
        QDialog.__init__(self,  parent)
        self.setupUi(self)   
        self.product=product
        self.mem=mem
        self.lblInvestment.setText("{0} ({1})".format(self.product.name,  self.product.id))
        self.quote=quote

        if quote==None:#Insert
            if self.product.type.id in (eProductType.Fund, eProductType.PensionPlan):
                self.chkNone.setCheckState(Qt.Checked)       
            else:
                if self.mem.localzone.now()>=self.product.stockmarket.today_closes():#Si ya ha cerrado la bolsa
                    today_closes=dtaware_changes_tz(self.product.stockmarket.today_closes(), self.mem.localzone.name)
                    self.wdgDT.setCombine(self.mem,  today_closes.date(), today_closes.time(),  self.mem.localzone)
                else:
                    self.wdgDT.set(self.mem)
        else:#Update
            self.wdgDT.set(self.mem, quote.datetime, self.mem.localzone)
            if self.quote.datetime.microsecond!=5:
                self.chkCanBePurged.setCheckState(Qt.Unchecked)
            self.wdgDT.setEnabled(False)
            self.chkNone.setEnabled(False)

    def on_chkNone_stateChanged(self, state):
        if state==Qt.Checked:      
            self.wdgDT.set(self.mem, dtaware(self.wdgDT.date(), self.product.stockmarket.closes, self.product.stockmarket.zone.name), self.product.stockmarket.zone)
            self.wdgDT.teTime.setEnabled(False)
            self.wdgDT.cmbZone.setEnabled(False)
            self.wdgDT.cmdNow.setEnabled(False)
            self.wdgDT.teMicroseconds.setEnabled(False)
        else:
            self.wdgDT.teTime.setEnabled(True)
            self.wdgDT.cmbZone.setEnabled(True)
            self.wdgDT.cmdNow.setEnabled(True)
            self.wdgDT.teMicroseconds.setEnabled(True)

    @pyqtSlot()
    def on_buttonbox_accepted(self):
        if not self.txtQuote.isValid():
            qmessagebox(self.tr("Incorrect data. Try again."))
            return
        if self.quote==None:#insert
            if self.chkCanBePurged.checkState()==Qt.Unchecked:#No puede ser purgado
                self.wdgDT.teMicroseconds.setValue(5)
            self.quote=Quote(self.mem).init__create(self.product, self.wdgDT.datetime(), self.txtQuote.decimal())
            self.quote.save()
        else:#update
            self.quote.quote=self.txtQuote.decimal()
            self.quote.save()
        self.product.needStatus(1, downgrade_to=0)
        self.mem.con.commit()
        self.accept()

    @pyqtSlot()
    def on_buttonbox_rejected(self):
        self.reject()#No haría falta pero para recordar que hay buttonbox



from PyQt5.QtWidgets import QWidget
from xulpymoney.ui.Ui_wdgCuriosity import Ui_wdgCuriosity

class wdgCuriosity(QWidget, Ui_wdgCuriosity):
    def __init__(self, mem,  parent = None):
        QWidget.__init__(self, parent)
        self.setupUi(self)
        self.mem=mem

    def setTitle(self, text):
        self.lblTitle.setText(text)
        
    def setText(self, text):
        self.lbl.setText(text)



from datetime import datetime
from urllib.request import urlopen
from json import loads

## Get Github file modification datetime
## https://api.github.com/repos/turulomio/xulpymoney/commits?path=products.xlsx
def get_file_modification_dt(user,project,path):
    url="https://api.github.com/repos/{}/{}/commits?path={}".format(user,project,path)
    bytes_j = urlopen(url).read()
    j=loads(bytes_j.decode('UTF-8'))
    return datetime.strptime(j[0]['commit']['author']['date'], "%Y-%m-%dT%H:%M:%SZ")

if __name__ == '__main__':
    print(get_file_modification_dt("turulomio","xulpymoney","products.xlsx"))


import datetime
from xulpymoney.libxulpymoney import Assets, InvestmentOperation, InvestmentOperationCurrentHeterogeneusManager, InvestmentOperationHeterogeneusManager
from PyQt5.QtCore import pyqtSlot
from PyQt5.QtGui import QIcon
from PyQt5.QtWidgets import QMenu, QWidget
from xulpymoney.ui.Ui_wdgInvestmentsOperations import Ui_wdgInvestmentsOperations
from xulpymoney.ui.frmInvestmentReport import frmInvestmentReport
from xulpymoney.ui.frmInvestmentOperationsAdd import frmInvestmentOperationsAdd
from xulpymoney.ui.frmAccountsReport import frmAccountsReport
from xulpymoney.ui.frmProductReport import frmProductReport

class wdgInvestmentsOperations(QWidget, Ui_wdgInvestmentsOperations):
    def __init__(self, mem,  parent=None):
        QWidget.__init__(self, parent)
        self.setupUi(self)
        self.mem=mem
        fechainicio=Assets(self.mem).first_datetime_with_user_data()         

         
        self.wy.initiate(fechainicio.year, datetime.date.today().year, datetime.date.today().year)
        self.wy.changed.connect(self.on_wy_mychanged)
        self.wy.label.hide()
        self.wy.hide()
        self.wym.initiate(fechainicio.year, datetime.date.today().year, datetime.date.today().year,  datetime.date.today().month)
        self.wym.changed.connect(self.on_wym_mychanged)
        self.wym.label.hide()
        self.setOperations=InvestmentOperationHeterogeneusManager(self.mem)
        self.setCurrent=InvestmentOperationCurrentHeterogeneusManager(self.mem)
        self.selOperation=None#For table
        self.selCurrentOperation=None#For tblCurrent
        self.table.settings(self.mem,  "wdgInvestmentsOperations")
        self.tblCurrent.settings(self.mem, "wdgInvestmentsOperations")
        self.tab.setCurrentIndex(0)
        self.load()
        self.load_current()
        
    def load(self):
        del self.setOperations.arr
        self.setOperations.arr=[]
        cur=self.mem.con.cursor()
        filters=""
        if self.cmbFilters.currentIndex()==0:#All
            filters=""
        elif self.cmbFilters.currentIndex()==1:#Purchasing
            filters=" and id_tiposoperaciones in (4)"
        elif self.cmbFilters.currentIndex()==2:#Purchasing
            filters=" and id_tiposoperaciones in (5)"
        elif self.cmbFilters.currentIndex()==3:#Purchasing
            filters=" and id_tiposoperaciones not in (4, 5)"
        
        
        if self.radYear.isChecked()==True:
            cur.execute("select * from operinversiones where date_part('year',datetime)=%s "+filters+" order by datetime",(self.wy.year,  ) )
        else:
            cur.execute("select * from operinversiones where date_part('year',datetime)=%s and date_part('month',datetime)=%s "+filters+" order by datetime",(self.wym.year, self.wym.month) )
        for row in cur:
            self.setOperations.append(InvestmentOperation(self.mem).init__db_row(row, self.mem.data.investments.find_by_id(row['id_inversiones']), self.mem.tiposoperaciones.find_by_id(row['id_tiposoperaciones'])))
        cur.close()
        
        self.setOperations.myqtablewidget(self.table)
        
    def load_current(self):
        for inv in self.mem.data.investments_active().arr:
            for o in inv.op_actual.arr:
                self.setCurrent.append(o)
        self.setCurrent.order_by_datetime()
        self.setCurrent.myqtablewidget(self.tblCurrent)

        
    @pyqtSlot(int) 
    def on_cmbFilters_currentIndexChanged(self, index):
        self.load()
        
    def on_radYear_toggled(self, toggle):
        if toggle==True:
            self.wym.hide()
            self.wy.show()
        else:
            self.wym.show()
            self.wy.hide()
        self.load()
        
    def on_wym_mychanged(self):
        self.load()            
    def on_wy_mychanged(self):
        self.load()    

    def on_table_itemSelectionChanged(self):
        self.selOperation=None
        for i in self.table.selectedItems():#itera por cada item no row.
            self.selOperation=self.setOperations.arr[i.row()]
            
    def on_tblCurrent_itemSelectionChanged(self):
        try: #Due it has one more row and crashes
            for i in self.tblCurrent.selectedItems():#itera por cada item no row.
                self.selCurrentOperation=self.setCurrent.arr[i.row()]
        except:
            self.selCurrentOperation=None
            
    
    @pyqtSlot() 
    def on_actionShowAccount_triggered(self):
        if self.tab.currentIndex()==0:#Operation list
            cuenta=self.selOperation.investment.account
        else:#Current operation list
            cuenta=self.selCurrentOperation.investment.account
        w=frmAccountsReport(self.mem, cuenta, self)
        w.exec_()
        self.load()
        
    @pyqtSlot() 
    def on_actionShowInvestment_triggered(self):
        if self.tab.currentIndex()==0:#Operation list
            investment=self.selOperation.investment
        else:#Current operation list
            investment=self.selCurrentOperation.investment
        w=frmInvestmentReport(self.mem, investment, self)
        w.exec_()
        self.load()
                
    @pyqtSlot() 
    def on_actionShowProduct_triggered(self):
        if self.tab.currentIndex()==0:#Operation list
            investment=self.selOperation.investment
        else:#Current operation list
            investment=self.selCurrentOperation.investment
        w=frmProductReport(self.mem, investment.product, investment, self)
        w.exec_()
        self.load()
                
    @pyqtSlot() 
    def on_actionRangeReport_triggered(self):
        self.selOperation.show_in_ranges= not self.selOperation.show_in_ranges
        self.selOperation.save()
        self.mem.con.commit()
        #self.selOperation doesn't belong to self.mem.data, it's a set of this widget, so I need to reload investment of the self.mem.data
        self.mem.data.investments.find_by_id(self.selOperation.investment.id).get_operinversiones()
        self.load()
        
    @pyqtSlot() 
    def on_actionShowInvestmentOperation_triggered(self):
        if self.tab.currentIndex()==0:#Operation list
            operation=self.selOperation
        else:#Current operation list
            operation=self.selCurrentOperation
        w=frmInvestmentOperationsAdd(self.mem, operation.investment, operation, self)
        w.exec_()
        self.load()
        

    def on_table_customContextMenuRequested(self,  pos):
        self.actionShowAccount.setEnabled(False)
        self.actionShowInvestment.setEnabled(False)
        self.actionShowInvestmentOperation.setEnabled(False)
        self.actionShowProduct.setEnabled(False)
        if self.selOperation!=None:
            if self.selOperation.investment.account.active==True:#only enabled if it's active
                self.actionShowAccount.setEnabled(True)
            if self.selOperation.investment.active==True:
                self.actionShowInvestment.setEnabled(True)
                self.actionShowInvestmentOperation.setEnabled(True)
            self.actionShowProduct.setEnabled(True)
            if self.selOperation.show_in_ranges==True:
                self.actionRangeReport.setText(self.tr("Hide in range report"))
                self.actionRangeReport.setIcon(QIcon(":/xulpymoney/eye_red.png"))
            else:
                self.actionRangeReport.setText(self.tr("Show in range report"))
                self.actionRangeReport.setIcon(QIcon(":/xulpymoney/eye.png"))

        menu=QMenu()
        menu.addAction(self.actionShowAccount)   
        menu.addSeparator()   
        menu.addAction(self.actionShowInvestment)   
        menu.addSeparator()   
        menu.addAction(self.actionShowInvestmentOperation)      
        menu.addSeparator()
        menu.addAction(self.actionShowProduct)
        menu.addSeparator()
        menu.addAction(self.actionRangeReport)
        menu.exec_(self.table.mapToGlobal(pos))
                
    def on_tblCurrent_customContextMenuRequested(self,  pos):
        if self.selCurrentOperation==None:
            self.actionShowAccount.setEnabled(False)
            self.actionShowInvestment.setEnabled(False)
            self.actionShowProduct.setEnabled(False)
        else:
            self.actionShowAccount.setEnabled(True)
            self.actionShowInvestment.setEnabled(True)
            self.actionShowProduct.setEnabled(True)

        menu=QMenu()
        menu.addAction(self.actionShowAccount)   
        menu.addSeparator()   
        menu.addAction(self.actionShowInvestment)   
        menu.addSeparator()
        menu.addAction(self.actionShowProduct)
        menu.exec_(self.tblCurrent.mapToGlobal(pos))
        
        



from PyQt5.QtCore import pyqtSignal, pyqtSlot
from PyQt5.QtWidgets import QWidget
import datetime
from xulpymoney.ui.Ui_wdgYear import Ui_wdgYear
from xulpymoney.libxulpymoneyfunctions import function_name, qmessagebox

class wdgYear(QWidget, Ui_wdgYear):
    changed=pyqtSignal()
    def __init__(self,  parent = None, name = None):
        QWidget.__init__(self,  parent)
        self.setupUi(self)
        
        
    def initiate(self, firstyear,  lastyear, currentyear):
        """Debe ser la primera función después del constructor"""
        if firstyear==None:
            self.setEnabled(False)
            print (function_name(self), "Firstyear is None")
            return
        
        self.firstyear=firstyear
        self.lastyear=lastyear
        for year in range(firstyear, lastyear+1):
            self.cmbYear.addItem(str(year), year)
            
        if currentyear<firstyear:#If I set a currentyear that doesn't exist
            self.set(firstyear)
        else:
            self.set(currentyear)
        
    def set(self,  year):
        self.year=year
        self.cmbYear.setCurrentIndex(self.year-self.firstyear)
        if self.year==self.lastyear:
            self.cmdNext.setEnabled(False)
        else:
            self.cmdNext.setEnabled(True)

        if self.year==self.firstyear:
            self.cmdPrevious.setEnabled(False)
        else:
            self.cmdPrevious.setEnabled(True)
    @pyqtSlot(str)      
    def on_cmbYear_currentIndexChanged(self, text):
        self.year=int(text)
        self.changed.emit()
        
       
    def on_cmdNext_pressed(self):
        if self.year==self.lastyear:
            qmessagebox(self.tr("I can't show the next year"))
            return
        self.year=self.year+1
        self.set(self.year)
        
    def on_cmdPrevious_pressed(self):
        if self.firstyear==self.year:
            qmessagebox(self.tr("I can't show the previous year"))
            return
        self.year=self.year-1
        self.set(self.year)
        
    @pyqtSlot()      
    def on_cmdCurrent_pressed(self):
        self.set(datetime.date.today().year)
        
        




## DO NOT DELETE. CONCEPT FUNCTION TO PROCESS OPERATIONS
## NECESITADO PARA DEPURAR
def have_same_sign(number1, number2):
    if (is_positive(number1)==True and is_positive(number2)==True) or (is_positive(number1)==False and is_positive(number2)==False):
        return True
    return False

def is_positive(number):
    if number>=0:
        return True
    return False

def set_sign_of_other_number(number, number_to_change):
    if is_positive(number):
       return abs(number_to_change)
    return -abs(number_to_change)

def process(op):
    cur=[]
    hst=[]

    print("Processing {}. Su suma es {}".format(op, sum(op)))

    for position, io in enumerate(op):
        if len(cur)==0 or have_same_sign(cur[0], io)==True:
            cur.append(io)
        elif have_same_sign(cur[0], io)==False:
            rest=io
            while rest!=0:
                if len(cur)>0:
                    if abs(cur[0])>=abs(rest):
                        hst.append(set_sign_of_other_number(io,rest))
                        if rest+cur[0]!=0:
                            cur.insert(0, rest+cur[0])
                            cur.pop(1)
                        else:
                            cur.pop(0)
                        rest=0
                        break
                    else: #Mayor el resto
                        hst.append(set_sign_of_other_number(io,cur[0]))
                        rest=rest+cur[0]
                        rest=set_sign_of_other_number(io,rest)
                        cur.pop(0)
#                        print("   REST>", rest, "Current", cur, "Historical", hst)
                else:
                    cur.insert(0, rest)
                    break
        print("  + IO", io, "Current", cur, "Historical", hst)
    print("La suma de Current es", sum(cur))
    print("")

process([1,-1,-2,2,3,4,-7,-3,-4,7]) #Exacto
process([1,-2,2,-3,4]) #Cambio signo
process([1,1,-3, -1, 5]) #Cambio signo generando 2 hst
process([529, 1597, 12, 43, 340, 40, 468, 502, 908, 1136, 17, 1118, 114, 122, 147, 588, 646, 725, 1136, 1486, 1935, 2027, 793, 248, 308, -602, -387])



## @brief Package to manage postgresql connection functions with qt timeouts
## THIS IS FROM XULPYMONEY PACKAGE IF YOU NEED THIS MODULE PLEASE SYNC IT FROM THERE, FOR EXAMPLE
## @code
##       print ("Copying libmanagers.py from Xulpymoney project")
##        os.chdir("your directory)
##        os.remove("connection_pg.py")
##        os.system("wget https://raw.githubusercontent.com/Turulomio/xulpymoney/master/xulpymoney/connection_pg.py  --no-clobber")
##        os.system("sed -i -e '3i ## THIS FILE HAS BEEN DOWNLOADED AT {} FROM https://github.com/Turulomio/xulpymoney/xulpymoney/connection_pg.py.' connection_pg.py".format(datetime.datetime.now()))
## @encode

import datetime
from PyQt5.QtCore import QObject,  pyqtSignal,  QTimer
from .connection_pg import Connection

class ConnectionQt(QObject,Connection):
    inactivity_timeout=pyqtSignal()
    def __init__(self):
        Connection.__init__(self)
        QObject.__init__(self)
        self.restart_timeout()
        self.inactivity_timeout_minutes=30

    def _check_inactivity(self):
        if datetime.datetime.now()-self._lastuse>datetime.timedelta(minutes=self.inactivity_timeout_minutes):
            self.disconnect()
            self._timerlastuse.stop()
            self.inactivity_timeout.emit()
        print ("Remaining time {}".format(self._lastuse+datetime.timedelta(minutes=self.inactivity_timeout_minutes)-datetime.datetime.now()))

    def cursor(self):
        self.restart_timeout()#Datetime who saves the las use of connection
        return self._con.cursor()

    def restart_timeout(self):
        """Resets timeout, usefull in long process without database connections"""
        self._lastuse=datetime.datetime.now()

    def cursor_one_row(self, sql, arr=[]):
        """Returns only one row"""
        self.restart_timeout()
        return super().cursor_one_row(sql, arr)

    def cursor_one_column(self, sql, arr=[]):
        """Returns un array with the results of the column"""
        self.restart_timeout()
        return super().cursor_one_column(sql, arr)

    def cursor_one_field(self, sql, arr=[]):
        """Returns only one field"""
        self.restart_timeout()
        return super().cursor_one_field(sql, arr)

        
    def connect(self, connection_string=None):
        """Used in code to connect using last self.strcon"""
        super().connect(connection_string)
        self.restart_timeout()
        self._timerlastuse = QTimer()
        self._timerlastuse.timeout.connect(self._check_inactivity)
        self._timerlastuse.start(300000)

    def disconnect(self):
        self._active=False
        if self._timerlastuse.isActive()==True:
            self._timerlastuse.stop()
        self._con.close()

